# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# Tests access private members of SDK classes to assert correct configuration.
# pylint: disable=protected-access

import unittest
from unittest.mock import patch

from opentelemetry.configuration._sdk import configure_sdk
from opentelemetry.configuration.models import (
    OpenTelemetryConfiguration,
)
from opentelemetry.configuration.models import (
    Propagator as PropagatorConfig,
)
from opentelemetry.configuration.models import (
    Resource as ResourceConfig,
)
from opentelemetry.configuration.models import (
    SimpleSpanProcessor as SimpleSpanProcessorConfig,
)
from opentelemetry.configuration.models import (
    SpanExporter as SpanExporterConfig,
)
from opentelemetry.configuration.models import (
    SpanProcessor as SpanProcessorConfig,
)
from opentelemetry.configuration.models import (
    TracerProvider as TracerProviderConfig,
)
from opentelemetry.sdk.trace import TracerProvider as SdkTracerProvider

_MIN_CONFIG_KWARGS = {"file_format": "1.0"}


def _config(**kwargs) -> OpenTelemetryConfiguration:
    return OpenTelemetryConfiguration(**{**_MIN_CONFIG_KWARGS, **kwargs})


class TestConfigureSdk(unittest.TestCase):
    @patch("opentelemetry.configuration._sdk.configure_propagator")
    @patch("opentelemetry.configuration._sdk.configure_logger_provider")
    @patch("opentelemetry.configuration._sdk.configure_meter_provider")
    @patch("opentelemetry.configuration._sdk.configure_tracer_provider")
    @patch("opentelemetry.configuration._sdk.create_resource")
    # pylint: disable=no-self-use
    def test_calls_each_signal_with_resource(
        self,
        mock_create_resource,
        mock_tracer,
        mock_meter,
        mock_logger,
        mock_propagator,
    ):
        sentinel_resource = object()
        mock_create_resource.return_value = sentinel_resource

        resource_cfg = ResourceConfig()
        tracer_cfg = TracerProviderConfig(processors=[])
        propagator_cfg = PropagatorConfig()
        config = _config(
            resource=resource_cfg,
            tracer_provider=tracer_cfg,
            propagator=propagator_cfg,
        )

        configure_sdk(config)

        mock_create_resource.assert_called_once_with(resource_cfg)
        mock_tracer.assert_called_once_with(tracer_cfg, sentinel_resource)
        mock_meter.assert_called_once_with(None, sentinel_resource)
        mock_logger.assert_called_once_with(None, sentinel_resource)
        mock_propagator.assert_called_once_with(propagator_cfg)

    @patch("opentelemetry.configuration._sdk.configure_propagator")
    @patch("opentelemetry.configuration._sdk.configure_logger_provider")
    @patch("opentelemetry.configuration._sdk.configure_meter_provider")
    @patch("opentelemetry.configuration._sdk.configure_tracer_provider")
    @patch("opentelemetry.configuration._sdk.create_resource")
    # pylint: disable=no-self-use
    def test_disabled_skips_everything(
        self,
        mock_create_resource,
        mock_tracer,
        mock_meter,
        mock_logger,
        mock_propagator,
    ):
        config = _config(
            disabled=True,
            tracer_provider=TracerProviderConfig(processors=[]),
        )

        configure_sdk(config)

        mock_create_resource.assert_not_called()
        mock_tracer.assert_not_called()
        mock_meter.assert_not_called()
        mock_logger.assert_not_called()
        mock_propagator.assert_not_called()

    @patch("opentelemetry.configuration._sdk.configure_propagator")
    @patch("opentelemetry.configuration._sdk.configure_logger_provider")
    @patch("opentelemetry.configuration._sdk.configure_meter_provider")
    @patch("opentelemetry.configuration._sdk.configure_tracer_provider")
    @patch("opentelemetry.configuration._sdk.create_resource")
    def test_absent_sections_pass_none(
        self,
        mock_create_resource,  # noqa: ARG002
        mock_tracer,
        mock_meter,
        mock_logger,
        mock_propagator,
    ):
        configure_sdk(_config())

        # Each configure_* is called exactly once, with config=None.
        self.assertEqual(mock_tracer.call_args.args[0], None)
        self.assertEqual(mock_meter.call_args.args[0], None)
        self.assertEqual(mock_logger.call_args.args[0], None)
        self.assertEqual(mock_propagator.call_args.args[0], None)


class TestConfigureSdkIntegration(unittest.TestCase):
    """End-to-end: build a real OpenTelemetryConfiguration and apply it."""

    @patch(
        "opentelemetry.configuration._tracer_provider.trace.set_tracer_provider"
    )
    def test_applies_tracer_provider_globally(self, mock_set_tracer):
        config = _config(
            tracer_provider=TracerProviderConfig(
                processors=[
                    SpanProcessorConfig(
                        simple=SimpleSpanProcessorConfig(
                            exporter=SpanExporterConfig(console={})
                        )
                    )
                ]
            )
        )

        configure_sdk(config)

        mock_set_tracer.assert_called_once()
        self.assertIsInstance(
            mock_set_tracer.call_args[0][0], SdkTracerProvider
        )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0


from opentelemetry.sdk.metrics._internal.aggregation import (
    AggregationTemporality,
)
from opentelemetry.sdk.metrics._internal.export import (
    ConsoleMetricExporter,
    InMemoryMetricReader,
    MetricExporter,
    MetricExportResult,
    MetricReader,
    PeriodicExportingMetricReader,
)

# The point module is not in the export directory to avoid a circular import.
from opentelemetry.sdk.metrics._internal.point import (  # noqa: F401
    Buckets,
    DataPointT,
    DataT,
    ExponentialHistogram,
    ExponentialHistogramDataPoint,
    Gauge,
    Histogram,
    HistogramDataPoint,
    Metric,
    MetricsData,
    NumberDataPoint,
    ResourceMetrics,
    ScopeMetrics,
    Sum,
)

__all__ = [
    "AggregationTemporality",
    "Buckets",
    "ConsoleMetricExporter",
    "InMemoryMetricReader",
    "MetricExporter",
    "MetricExportResult",
    "MetricReader",
    "PeriodicExportingMetricReader",
    "DataPointT",
    "DataT",
    "ExponentialHistogram",
    "ExponentialHistogramDataPoint",
    "Gauge",
    "Histogram",
    "HistogramDataPoint",
    "Metric",
    "MetricsData",
    "NumberDataPoint",
    "ResourceMetrics",
    "ScopeMetrics",
    "Sum",
]



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import threading
import tracemalloc
from functools import lru_cache

import pytest

from opentelemetry.attributes import BoundedAttributes
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import (
    ReadableSpan,
    SpanProcessor,
    TracerProvider,
    _default_tracer_configurator,
    _RuleBasedTracerConfigurator,
    _TracerConfig,
    sampling,
)
from opentelemetry.sdk.trace.export import BatchSpanProcessor
from opentelemetry.sdk.trace.export.in_memory_span_exporter import (
    InMemorySpanExporter,
)
from opentelemetry.sdk.util import BoundedList
from opentelemetry.sdk.util.instrumentation import _scope_name_matches_glob
from opentelemetry.trace import SpanContext, TraceFlags

tracer_provider = TracerProvider(
    sampler=sampling.DEFAULT_ON,
    resource=Resource(
        {
            "service.name": "A123456789",
            "service.version": "1.34567890",
            "service.instance.id": "123ab456-a123-12ab-12ab-12340a1abc12",
        }
    ),
)
tracer = tracer_provider.get_tracer("sdk_tracer_provider")


@pytest.fixture(params=[0, 1, 10, 50])
def num_tracer_configurator_rules(request):
    return request.param


def test_simple_start_span(benchmark):
    def benchmark_start_span():
        span = tracer.start_span(
            "benchmarkedSpan",
            attributes={"long.attribute": -10000000001000000000},
        )
        span.add_event("benchmarkEvent")
        span.end()

    benchmark(benchmark_start_span)


@pytest.mark.parametrize("num_attrs", [0, 1, 10, 50, 128])
def test_start_span_with_attributes(benchmark, num_attrs):
    attrs = {f"key{i}": f"value{i}" for i in range(num_attrs)}

    def benchmark_start_span():
        span = tracer.start_span("benchmarkedSpan", attributes=attrs)
        span.end()

    benchmark(benchmark_start_span)


# pylint: disable=protected-access,redefined-outer-name
def test_simple_start_span_with_tracer_configurator_rules(
    benchmark, num_tracer_configurator_rules
):
    def benchmark_start_span():
        span = tracer.start_span(
            "benchmarkedSpan",
            attributes={"long.attribute": -10000000001000000000},
        )
        span.add_event("benchmarkEvent")
        span.end()

    @lru_cache
    def tracer_configurator(tracer_scope):
        # this is testing 100 rules that is an extreme case
        return _RuleBasedTracerConfigurator(
            rules=[
                (
                    _scope_name_matches_glob(glob_pattern=str(i)),
                    _TracerConfig(is_enabled=True),
                )
                for i in range(num_tracer_configurator_rules)
            ],
            default_config=_TracerConfig(is_enabled=True),
        )(tracer_scope)

    tracer_provider._set_tracer_configurator(
        tracer_configurator=tracer_configurator
    )
    benchmark(benchmark_start_span)
    tracer_provider._set_tracer_configurator(
        tracer_configurator=_default_tracer_configurator
    )


@pytest.mark.parametrize("num_attrs", [1, 10, 50, 128])
def test_set_attribute(benchmark, num_attrs):
    attrs = {f"key{i}": f"value{i}" for i in range(num_attrs)}

    def benchmark_set_attribute():
        span = tracer.start_span("benchmarkedSpan")
        for key, value in attrs.items():
            span.set_attribute(key, value)
        span.end()

    benchmark(benchmark_set_attribute)


@pytest.mark.parametrize(
    "attr_type,value",
    [
        ("bool", True),
        ("int", 42),
        ("float", 3.14),
        ("str", "hello world"),
        ("bytes", b"hello world"),
        ("seq_bool", (True, False, True)),
        ("seq_int", (1, 2, 3, 4, 5)),
        ("seq_float", (1.1, 2.2, 3.3)),
        ("seq_str", ("a", "b", "c", "d", "e")),
        ("seq_bytes", (b"a", b"b", b"c")),
    ],
)
def test_set_attribute_types(benchmark, attr_type, value):
    attrs = {f"key{i}": value for i in range(128)}

    def benchmark_set_attribute():
        for _ in range(5_000):
            span = tracer.start_span("benchmarkedSpan")
            for key, val in attrs.items():
                span.set_attribute(key, val)
            span.end()

    benchmark(benchmark_set_attribute)


def test_simple_start_as_current_span(benchmark):
    def benchmark_start_as_current_span():
        with tracer.start_as_current_span(
            "benchmarkedSpan",
            attributes={"long.attribute": -10000000001000000000},
        ) as span:
            span.add_event("benchmarkEvent")

    benchmark(benchmark_start_as_current_span)


class _EventsReadingProcessor(SpanProcessor):
    def on_end(self, span: ReadableSpan) -> None:
        _ = span.events


class _LinksReadingProcessor(SpanProcessor):
    def on_end(self, span: ReadableSpan) -> None:
        _ = span.links


_link_context = SpanContext(
    trace_id=0x000000000000000000000000DEADBEEF,
    span_id=0x00000000DEADBEF0,
    is_remote=True,
    trace_flags=TraceFlags(TraceFlags.SAMPLED),
)


@pytest.mark.parametrize("num_events", [1, 10, 50, 128])
def test_read_events(benchmark, num_events):
    provider = TracerProvider(sampler=sampling.DEFAULT_ON)
    provider.add_span_processor(_EventsReadingProcessor())
    tp = provider.get_tracer("bench")

    peaks = []
    for _ in range(200):
        span = tp.start_span("benchmarkedSpan")
        for event in range(num_events):
            span.add_event(f"event{event}", {"k": "v"})
        tracemalloc.start()
        span.end()
        _, peak = tracemalloc.get_traced_memory()
        tracemalloc.stop()
        peaks.append(peak)
    benchmark.extra_info["mean_alloc_bytes"] = sum(peaks) / len(peaks)

    def benchmark_read_events():
        span = tp.start_span("benchmarkedSpan")
        for event in range(num_events):
            span.add_event(f"event{event}", {"k": "v"})
        span.end()

    benchmark(benchmark_read_events)


@pytest.mark.parametrize("num_links", [1, 10, 50, 128])
def test_read_links(benchmark, num_links):
    provider = TracerProvider(sampler=sampling.DEFAULT_ON)
    provider.add_span_processor(_LinksReadingProcessor())
    tp = provider.get_tracer("bench")

    peaks = []
    for _ in range(200):
        span = tp.start_span("benchmarkedSpan")
        for _ in range(num_links):
            span.add_link(_link_context)
        tracemalloc.start()
        span.end()
        _, peak = tracemalloc.get_traced_memory()
        tracemalloc.stop()
        peaks.append(peak)
    benchmark.extra_info["mean_alloc_bytes"] = sum(peaks) / len(peaks)

    def benchmark_read_links():
        span = tp.start_span("benchmarkedSpan")
        for _ in range(num_links):
            span.add_link(_link_context)
        span.end()

    benchmark(benchmark_read_links)


@pytest.mark.parametrize("num_attrs", [1, 10, 50, 128])
def test_bounded_attribute_iterator(benchmark, num_attrs):
    attrs = BoundedAttributes(
        attributes={f"key{i}": f"value{i}" for i in range(num_attrs)}
    )

    peaks = []
    for _ in range(200):
        tracemalloc.start()
        list(attrs)
        _, peak = tracemalloc.get_traced_memory()
        tracemalloc.stop()
        peaks.append(peak)
    benchmark.extra_info["mean_alloc_bytes"] = sum(peaks) / len(peaks)

    def benchmark_iter():
        list(attrs)

    benchmark(benchmark_iter)


@pytest.mark.parametrize("num_items", [1, 10, 50, 128])
def test_bounded_list_iterator(benchmark, num_items):
    blist = BoundedList.from_seq(num_items, range(num_items))

    peaks = []
    for _ in range(200):
        tracemalloc.start()
        list(blist)
        _, peak = tracemalloc.get_traced_memory()
        tracemalloc.stop()
        peaks.append(peak)
    benchmark.extra_info["mean_alloc_bytes"] = sum(peaks) / len(peaks)

    def benchmark_iter():
        list(blist)

    benchmark(benchmark_iter)


# Total span work is fixed at TOTAL_SPANS regardless of thread count so that
# ideal (GIL-free) parallelism would produce a flat wall-clock time across
# thread counts. Any increase instead reveals GIL + lock contention.
_TOTAL_SPANS = 128
_ATTRS_PER_SPAN = {f"key{i}": f"value{i}" for i in range(50)}


@pytest.mark.parametrize("num_threads", [1, 2, 4, 8])
def test_gil_contention_batch_processor(benchmark, num_threads):
    exporter = InMemorySpanExporter()
    provider = TracerProvider(sampler=sampling.DEFAULT_ON)
    # max_export_batch_size=16 ensures the export threshold is crossed
    # during the benchmark so _worker_awaken.set() contention is exercised.
    provider.add_span_processor(
        BatchSpanProcessor(exporter, max_export_batch_size=16)
    )
    tracer = provider.get_tracer("bench")
    spans_per_thread = _TOTAL_SPANS // num_threads

    def worker():
        for _ in range(spans_per_thread):
            span = tracer.start_span("s")
            span.set_attributes(_ATTRS_PER_SPAN)
            span.end()

    def benchmark_fn():
        threads = [threading.Thread(target=worker) for _ in range(num_threads)]
        for thread in threads:
            thread.start()
        for thread in threads:
            thread.join()

    benchmark(benchmark_fn)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# otcollector.py

from opentelemetry import trace
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import (
    OTLPSpanExporter,
)
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor

span_exporter = OTLPSpanExporter(
    # optional
    # endpoint="myCollectorURL:4317",
    # credentials=ChannelCredentials(credentials),
    # headers=(("metadata", "metadata")),
)
tracer_provider = TracerProvider()
trace.set_tracer_provider(tracer_provider)
span_processor = BatchSpanProcessor(span_exporter)
tracer_provider.add_span_processor(span_processor)

# Configure the tracer to use the collector exporter
tracer = trace.get_tracer_provider().get_tracer(__name__)

with tracer.start_as_current_span("foo"):
    print("Hello world!")



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum
from typing import Final

DISK_IO_DIRECTION: Final = "disk.io.direction"
"""
The disk IO operation direction.
"""


class DiskIoDirectionValues(Enum):
    READ = "read"
    """read."""
    WRITE = "write"
    """write."""



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from mysql.connector import connect

from opentelemetry import trace
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import (
    OTLPSpanExporter,
)
from opentelemetry.instrumentation.mysql import MySQLInstrumentor
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor

resource = Resource.create(
    attributes={
        "service.name": "sqlcommenter-example",
    }
)
trace.set_tracer_provider(TracerProvider(resource=resource))
span_processor = BatchSpanProcessor(
    OTLPSpanExporter(endpoint="http://localhost:4317")
)
trace.get_tracer_provider().add_span_processor(span_processor)

cnx = connect(
    host="localhost",
    port=3366,
    user="books",
    password="books123",
    database="books",
)

# Instruments MySQL queries with sqlcommenter enabled
# and comment-in-span-attribute enabled.
# Returns wrapped connection to generate traces.
cnx = MySQLInstrumentor().instrument_connection(
    connection=cnx,
    enable_commenter=True,
    enable_attribute_commenter=True,
)

cursor = cnx.cursor()
statement = "SELECT * FROM authors WHERE id = %s"

# Each SELECT query generates one mysql log with sqlcomment
# and one OTel span with `db.statement` attribute that also
# includes sqlcomment.
for cid in range(1, 4):
    cursor.execute(statement, (cid,))
    rows = cursor.fetchall()
    print(f"Found author: {rows[0]}")

print("Done.")



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import json
import unittest
import warnings

from opentelemetry._logs import LogRecord, SeverityNumber
from opentelemetry.attributes import BoundedAttributes
from opentelemetry.context import get_current
from opentelemetry.sdk._logs import (
    LogRecordDroppedAttributesWarning,
    LogRecordLimits,
    ReadableLogRecord,
    ReadWriteLogRecord,
)
from opentelemetry.sdk.resources import Resource
from opentelemetry.trace.span import TraceFlags


class TestLogRecord(unittest.TestCase):
    def test_log_record_to_json(self):
        log_record = ReadableLogRecord(
            LogRecord(
                timestamp=0,
                observed_timestamp=0,
                body={"key": "logLine", "bytes": b"123"},
                attributes={
                    "mapping": {"key": "value"},
                    "none": None,
                    "sequence": [1, 2],
                    "str": "string",
                },
                event_name="a.event",
            ),
            resource=Resource({"service.name": "foo"}),
        )

        self.assertEqual(
            log_record.to_json(indent=None),
            '{"body": {"key": "logLine", "bytes": "MTIz"}, "severity_number": null, "severity_text": null, "attributes": {"mapping": {"key": "value"}, "none": null, "sequence": [1, 2], "str": "string"}, "dropped_attributes": 0, "timestamp": "1970-01-01T00:00:00.000000Z", "observed_timestamp": "1970-01-01T00:00:00.000000Z", "trace_id": "0x00000000000000000000000000000000", "span_id": "0x0000000000000000", "trace_flags": 0, "resource": {"attributes": {"service.name": "foo"}, "schema_url": ""}, "event_name": "a.event"}',
        )

    def test_log_record_to_json_serializes_severity_number_as_int(self):
        actual = ReadableLogRecord(
            LogRecord(
                timestamp=0,
                severity_number=SeverityNumber.WARN,
                observed_timestamp=0,
                body="a log line",
            ),
            resource=Resource({"service.name": "foo"}),
        )

        decoded = json.loads(actual.to_json())
        self.assertEqual(SeverityNumber.WARN.value, decoded["severity_number"])

    def test_log_record_to_json_serializes_null_severity_number(self):
        actual = ReadableLogRecord(
            LogRecord(
                observed_timestamp=0,
                body="a log line",
            ),
            resource=Resource({"service.name": "foo"}),
        )

        decoded = json.loads(actual.to_json())
        self.assertEqual(None, decoded["timestamp"])

    def test_log_record_bounded_attributes(self):
        attr = {"key": "value"}

        result = ReadWriteLogRecord(
            LogRecord(timestamp=0, body="a log line", attributes=attr)
        )

        self.assertTrue(
            isinstance(result.log_record.attributes, BoundedAttributes)
        )

    def test_log_record_dropped_attributes_empty_limits(self):
        attr = {"key": "value"}

        result = ReadWriteLogRecord(
            LogRecord(timestamp=0, body="a log line", attributes=attr)
        )

        self.assertTrue(result.dropped_attributes == 0)

    def test_log_record_dropped_attributes_set_limits_max_attribute(self):
        attr = {"key": "value", "key2": "value2"}
        limits = LogRecordLimits(
            max_attributes=1,
        )

        result = ReadWriteLogRecord(
            LogRecord(timestamp=0, body="a log line", attributes=attr),
            limits=limits,
        )
        self.assertTrue(result.dropped_attributes == 1)

    def test_log_record_dropped_attributes_set_limits_max_attribute_length(
        self,
    ):
        attr = {"key": "value", "key2": "value2"}
        expected = {"key": "v", "key2": "v"}
        limits = LogRecordLimits(
            max_attribute_length=1,
        )

        result = ReadWriteLogRecord(
            LogRecord(
                timestamp=0,
                body="a log line",
                attributes=attr,
            ),
            limits=limits,
        )
        self.assertTrue(result.dropped_attributes == 0)
        self.assertEqual(expected, result.log_record.attributes)

    def test_log_record_dropped_attributes_set_limits(self):
        attr = {"key": "value", "key2": "value2"}
        expected = {"key2": "v"}
        limits = LogRecordLimits(
            max_attributes=1,
            max_attribute_length=1,
        )

        result = ReadWriteLogRecord(
            LogRecord(
                timestamp=0,
                body="a log line",
                attributes=attr,
            ),
            limits=limits,
        )
        self.assertTrue(result.dropped_attributes == 1)
        self.assertEqual(expected, result.log_record.attributes)

    def test_log_record_dropped_attributes_set_limits_warning_once(self):
        attr = {"key1": "value1", "key2": "value2"}
        limits = LogRecordLimits(
            max_attributes=1,
            max_attribute_length=1,
        )

        with warnings.catch_warnings(record=True) as cw:
            for _ in range(10):
                ReadWriteLogRecord(
                    LogRecord(
                        timestamp=0,
                        body="a log line",
                        attributes=attr,
                    ),
                    limits=limits,
                )

        # Check that at least one LogRecordDroppedAttributesWarning was emitted
        dropped_attributes_warnings = [
            w
            for w in cw
            if isinstance(w.message, LogRecordDroppedAttributesWarning)
        ]
        self.assertEqual(
            len(dropped_attributes_warnings),
            1,
            "Expected exactly one LogRecordDroppedAttributesWarning due to simplefilter('once')",
        )

        # Check the message content of the LogRecordDroppedAttributesWarning
        warning_message = str(dropped_attributes_warnings[0].message)
        self.assertIn(
            "Log record attributes were dropped due to limits",
            warning_message,
        )

    def test_log_record_dropped_attributes_unset_limits(self):
        attr = {"key": "value", "key2": "value2"}
        limits = LogRecordLimits()

        result = ReadWriteLogRecord(
            LogRecord(
                timestamp=0,
                body="a log line",
                attributes=attr,
            ),
            limits=limits,
        )
        self.assertTrue(result.dropped_attributes == 0)
        self.assertEqual(attr, result.log_record.attributes)

    # pylint:disable=protected-access
    def test_log_record_from_api_log_record(self):
        api_log_record = LogRecord(
            timestamp=1,
            observed_timestamp=2,
            context=get_current(),
            severity_text="WARN",
            severity_number=SeverityNumber.WARN,
            body="a log line",
            attributes={"a": "b"},
            event_name="an.event",
        )

        resource = Resource.create({})
        record = ReadWriteLogRecord._from_api_log_record(
            record=api_log_record, resource=resource
        )

        self.assertEqual(record.log_record.timestamp, 1)
        self.assertEqual(record.log_record.observed_timestamp, 2)
        self.assertEqual(record.log_record.context, get_current())
        # trace_id, span_id, and trace_flags come from the context's span
        self.assertEqual(record.log_record.trace_id, 0)
        self.assertEqual(record.log_record.span_id, 0)
        self.assertEqual(record.log_record.trace_flags, TraceFlags(0x00))
        self.assertEqual(record.log_record.severity_text, "WARN")
        self.assertEqual(
            record.log_record.severity_number, SeverityNumber.WARN
        )
        self.assertEqual(record.log_record.body, "a log line")
        self.assertEqual(record.log_record.attributes, {"a": "b"})
        self.assertEqual(record.log_record.event_name, "an.event")
        self.assertEqual(record.resource, resource)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import json
import os
import subprocess
import sys
import unittest


class TestMetrics(unittest.TestCase):
    def test_metrics(self):
        """Test that metrics example produces expected values"""
        # Run the metrics example
        test_script = f"{os.path.dirname(os.path.realpath(__file__))}/../metrics_example.py"

        result = subprocess.run(
            [sys.executable, test_script],
            capture_output=True,
            text=True,
            timeout=10,
            check=True,
        )

        # Script should run successfully
        self.assertEqual(result.returncode, 0)

        # Parse the JSON output
        output_data = json.loads(result.stdout)

        # Get the metrics from the JSON structure
        metrics = output_data["resource_metrics"][0]["scope_metrics"][0][
            "metrics"
        ]

        # Create a lookup dict for easier testing
        metrics_by_name = {metric["name"]: metric for metric in metrics}

        # Test Counter: should be 1 (called counter.add(1))
        counter_value = metrics_by_name["counter"]["data"]["data_points"][0][
            "value"
        ]
        self.assertEqual(counter_value, 1, "Counter should have value 1")

        # Test UpDownCounter: should be -4 (1 + (-5) = -4)
        updown_value = metrics_by_name["updown_counter"]["data"][
            "data_points"
        ][0]["value"]
        self.assertEqual(
            updown_value, -4, "UpDownCounter should have value -4"
        )

        # Test Histogram: should have count=1, sum=99.9
        histogram_data = metrics_by_name["histogram"]["data"]["data_points"][0]
        self.assertEqual(
            histogram_data["count"], 1, "Histogram should have count 1"
        )
        self.assertEqual(
            histogram_data["sum"], 99.9, "Histogram should have sum 99.9"
        )

        # Test Gauge: should be 1 (last value set)
        gauge_value = metrics_by_name["gauge"]["data"]["data_points"][0][
            "value"
        ]
        self.assertEqual(gauge_value, 1, "Gauge should have value 1")

        # Test Observable Counter: should be 1 (from callback)
        obs_counter_value = metrics_by_name["observable_counter"]["data"][
            "data_points"
        ][0]["value"]
        self.assertEqual(
            obs_counter_value, 1, "Observable counter should have value 1"
        )

        # Test Observable UpDownCounter: should be -10 (from callback)
        obs_updown_value = metrics_by_name["observable_updown_counter"][
            "data"
        ]["data_points"][0]["value"]
        self.assertEqual(
            obs_updown_value,
            -10,
            "Observable updown counter should have value -10",
        )

        # Test Observable Gauge: should be 9 (from callback)
        obs_gauge_value = metrics_by_name["observable_gauge"]["data"][
            "data_points"
        ][0]["value"]
        self.assertEqual(
            obs_gauge_value, 9, "Observable gauge should have value 9"
        )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0
from typing import Any

import pytest  # type: ignore

from opentelemetry.codegen.json.writer import CodeWriter


def test_initialization() -> None:
    writer = CodeWriter(indent_size=2)
    # pylint: disable-next=protected-access
    assert writer._indent_size == 2
    assert not writer.to_lines()
    assert writer.to_string() == ""


def test_writeln_indentation() -> None:
    writer = CodeWriter(indent_size=4)
    writer.writeln("line1")
    with writer.indent():
        writer.writeln("line2")
        with writer.indent():
            writer.writeln("line3")
    writer.writeln("line4")

    expected = ["line1", "    line2", "        line3", "line4"]
    assert writer.to_lines() == expected


def test_writemany() -> None:
    writer = CodeWriter()
    writer.writemany("a", "b", "c")
    assert writer.to_lines() == ["a", "b", "c"]


@pytest.mark.parametrize(
    "content, expected",
    [
        ("single line", ["# single line"]),
        (["line1", "line2"], ["# line1", "# line2"]),
    ],
)
def test_comment(content: str, expected: list[str]) -> None:
    writer = CodeWriter()
    writer.comment(content)
    assert writer.to_lines() == expected


@pytest.mark.parametrize(
    "content, expected",
    [
        ("single line", ['"""single line"""']),
        (["line1", "line2"], ['"""', "line1", "line2", '"""']),
    ],
)
def test_docstring(content: str, expected: list[str]) -> None:
    writer = CodeWriter()
    writer.docstring(content)
    assert writer.to_lines() == expected


@pytest.mark.parametrize(
    "module, items, expected",
    [
        ("os", [], ["import os"]),
        ("typing", ["Any", "Optional"], ["from typing import Any, Optional"]),
    ],
)
def test_import(module: str, items: list[str], expected: list[str]) -> None:
    writer = CodeWriter()
    writer.import_(module, *items)
    assert writer.to_lines() == expected


def test_suite() -> None:
    writer = CodeWriter()
    with writer.block("def foo():"):
        writer.writeln("pass")
    assert writer.to_lines() == ["def foo():", "    pass"]


@pytest.mark.parametrize(
    "name, bases, decorators, expected",
    [
        ("MyClass", None, None, ["class MyClass:"]),
        ("MyClass", ["Base"], ["deco"], ["@deco", "class MyClass(Base):"]),
        (
            "MyClass",
            ["B1", "B2"],
            ["d1", "d2"],
            ["@d1", "@d2", "class MyClass(B1, B2):"],
        ),
    ],
)
def test_class(
    name: str,
    bases: str | None,
    decorators: list[str] | None,
    expected: list[str],
) -> None:
    writer = CodeWriter()
    with writer.class_(name, bases=bases, decorators=decorators):
        pass
    assert writer.to_lines() == expected


@pytest.mark.parametrize(
    "kwargs, expected",
    [
        ({"name": "DC"}, ["@dataclasses.dataclass", "class DC:"]),
        (
            {"name": "DC", "frozen": True},
            ["@dataclasses.dataclass(frozen=True)", "class DC:"],
        ),
        (
            {"name": "DC", "slots": True},
            ["@dataclasses.dataclass(slots=True)", "class DC:"],
        ),
        (
            {"name": "DC", "frozen": True, "slots": True},
            ["@dataclasses.dataclass(frozen=True, slots=True)", "class DC:"],
        ),
        (
            {"name": "DC", "decorator_name": "custom.dc"},
            ["@custom.dc", "class DC:"],
        ),
    ],
)
def test_dataclass(kwargs: dict[str, Any], expected: list[str]) -> None:
    writer = CodeWriter()
    with writer.dataclass(**kwargs):
        pass
    assert writer.to_lines() == expected


def test_enum() -> None:
    writer = CodeWriter()
    with writer.enum("MyEnum", bases=["IntEnum"]):
        writer.enum_member("A", 1)
        writer.enum_member("B", 2)
    expected = [
        "class MyEnum(enum.Enum, IntEnum):",
        "    A = 1",
        "    B = 2",
    ]
    assert writer.to_lines() == expected


@pytest.mark.parametrize(
    "name, type_hint, default, default_factory, expected",
    [
        ("x", "int", None, None, ["x: int"]),
        ("x", "int", 10, None, ["x: int = 10"]),
        (
            "x",
            "list",
            None,
            "list",
            ["x: list = dataclasses.field(default_factory=list)"],
        ),
    ],
)
def test_field(
    name: str,
    type_hint: str,
    default: Any | None,
    default_factory: Any | None,
    expected: list[str],
) -> None:
    writer = CodeWriter()
    writer.field(
        name, type_hint, default=default, default_factory=default_factory
    )
    assert writer.to_lines() == expected


def test_function() -> None:
    writer = CodeWriter()
    with writer.function("foo", ["a: int", "b: str"], return_type="bool"):
        writer.return_("True")
    expected = ["def foo(a: int, b: str) -> bool:", "    return True"]
    assert writer.to_lines() == expected


def test_control_flow() -> None:
    writer = CodeWriter()
    with writer.if_("a > b"):
        writer.pass_()
    with writer.elif_("a == b"):
        writer.pass_()
    with writer.else_():
        writer.pass_()

    expected = [
        "if a > b:",
        "    pass",
        "elif a == b:",
        "    pass",
        "else:",
        "    pass",
    ]
    assert writer.to_lines() == expected


def test_loops() -> None:
    writer = CodeWriter()
    with writer.for_("i", "range(10)"):
        writer.writeln("print(i)")
    with writer.while_("True"):
        writer.writeln("break")

    expected = [
        "for i in range(10):",
        "    print(i)",
        "while True:",
        "    break",
    ]
    assert writer.to_lines() == expected


def test_assignment_and_assertions() -> None:
    writer = CodeWriter()
    writer.assignment("x", "1", type_hint="int")

    expected = ["x: int = 1"]
    assert writer.to_lines() == expected


def test_method_alias():
    writer = CodeWriter()
    with writer.method("m", "self"):
        writer.pass_()
    assert "def m(self):" in writer.to_string()



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from opentracing.ext import tags

# pylint: disable=import-error
from ..utils import get_logger

logger = get_logger(__name__)


class RequestHandler:
    def __init__(self, tracer, context=None, ignore_active_span=True):
        self.tracer = tracer
        self.context = context
        self.ignore_active_span = ignore_active_span

    def before_request(self, request, request_context):
        logger.info("Before request %s", request)

        # If we should ignore the active Span, use any passed SpanContext
        # as the parent. Else, use the active one.
        if self.ignore_active_span:
            span = self.tracer.start_span(
                "send", child_of=self.context, ignore_active_span=True
            )
        else:
            span = self.tracer.start_span("send")

        span.set_tag(tags.SPAN_KIND, tags.SPAN_KIND_RPC_CLIENT)

        request_context["span"] = span

    def after_request(self, request, request_context):
        # pylint: disable=no-self-use
        logger.info("After request %s", request)

        span = request_context.get("span")
        if span is not None:
            span.finish()



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import unittest
from logging import ERROR

from opentelemetry import context


def do_work() -> None:
    context.attach(context.set_value("say", "bar"))


class ContextTestCases:
    class BaseTest(unittest.TestCase):
        def setUp(self) -> None:
            self.previous_context = context.get_current()

        def tearDown(self) -> None:
            context.attach(self.previous_context)

        def test_context(self):
            self.assertIsNone(context.get_value("say"))
            empty = context.get_current()
            second = context.set_value("say", "foo")

            self.assertEqual(context.get_value("say", context=second), "foo")

            do_work()
            self.assertEqual(context.get_value("say"), "bar")
            third = context.get_current()

            self.assertIsNone(context.get_value("say", context=empty))
            self.assertEqual(context.get_value("say", context=second), "foo")
            self.assertEqual(context.get_value("say", context=third), "bar")

        def test_set_value(self):
            first = context.set_value("a", "yyy")
            second = context.set_value("a", "zzz")
            third = context.set_value("a", "---", first)
            self.assertEqual("yyy", context.get_value("a", context=first))
            self.assertEqual("zzz", context.get_value("a", context=second))
            self.assertEqual("---", context.get_value("a", context=third))
            self.assertEqual(None, context.get_value("a"))

        def test_attach(self):
            context.attach(context.set_value("a", "yyy"))

            token = context.attach(context.set_value("a", "zzz"))
            self.assertEqual("zzz", context.get_value("a"))

            context.detach(token)
            self.assertEqual("yyy", context.get_value("a"))

            with self.assertLogs(level=ERROR):
                context.detach(token)

        def test_detach_out_of_order(self):
            t1 = context.attach(context.set_value("c", 1))
            self.assertEqual(context.get_current(), {"c": 1})
            t2 = context.attach(context.set_value("c", 2))
            self.assertEqual(context.get_current(), {"c": 2})
            context.detach(t1)
            self.assertEqual(context.get_current(), {})
            context.detach(t2)
            self.assertEqual(context.get_current(), {"c": 1})



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase

from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics._internal.aggregation import (
    _DEFAULT_EXPLICIT_BUCKET_HISTOGRAM_AGGREGATION_BOUNDARIES,
)
from opentelemetry.sdk.metrics._internal.instrument import Histogram
from opentelemetry.sdk.metrics.export import InMemoryMetricReader
from opentelemetry.sdk.metrics.view import (
    ExplicitBucketHistogramAggregation,
    View,
)


class TestHistogramAdvisory(TestCase):
    def test_default(self):
        reader = InMemoryMetricReader()
        meter_provider = MeterProvider(
            metric_readers=[reader],
        )
        meter = meter_provider.get_meter("testmeter")
        histogram = meter.create_histogram(
            "testhistogram",
            explicit_bucket_boundaries_advisory=[1.0, 2.0, 3.0],
        )
        histogram.record(1, {"label": "value"})
        histogram.record(2, {"label": "value"})
        histogram.record(3, {"label": "value"})

        metrics = reader.get_metrics_data()
        self.assertEqual(len(metrics.resource_metrics), 1)
        self.assertEqual(len(metrics.resource_metrics[0].scope_metrics), 1)
        self.assertEqual(
            len(metrics.resource_metrics[0].scope_metrics[0].metrics), 1
        )
        metric = metrics.resource_metrics[0].scope_metrics[0].metrics[0]
        self.assertEqual(metric.name, "testhistogram")
        self.assertEqual(
            metric.data.data_points[0].explicit_bounds, (1.0, 2.0, 3.0)
        )

    def test_empty_buckets(self):
        reader = InMemoryMetricReader()
        meter_provider = MeterProvider(
            metric_readers=[reader],
        )
        meter = meter_provider.get_meter("testmeter")
        histogram = meter.create_histogram(
            "testhistogram",
            explicit_bucket_boundaries_advisory=[],
        )
        histogram.record(1, {"label": "value"})
        histogram.record(2, {"label": "value"})
        histogram.record(3, {"label": "value"})

        metrics = reader.get_metrics_data()
        self.assertEqual(len(metrics.resource_metrics), 1)
        self.assertEqual(len(metrics.resource_metrics[0].scope_metrics), 1)
        self.assertEqual(
            len(metrics.resource_metrics[0].scope_metrics[0].metrics), 1
        )
        metric = metrics.resource_metrics[0].scope_metrics[0].metrics[0]
        self.assertEqual(metric.name, "testhistogram")
        self.assertEqual(metric.data.data_points[0].explicit_bounds, ())

    def test_view_default_aggregation(self):
        reader = InMemoryMetricReader()
        view = View(instrument_name="testhistogram")
        meter_provider = MeterProvider(
            metric_readers=[reader],
            views=[view],
        )
        meter = meter_provider.get_meter("testmeter")
        histogram = meter.create_histogram(
            "testhistogram",
            explicit_bucket_boundaries_advisory=[1.0, 2.0, 3.0],
        )
        histogram.record(1, {"label": "value"})
        histogram.record(2, {"label": "value"})
        histogram.record(3, {"label": "value"})

        metrics = reader.get_metrics_data()
        self.assertEqual(len(metrics.resource_metrics), 1)
        self.assertEqual(len(metrics.resource_metrics[0].scope_metrics), 1)
        self.assertEqual(
            len(metrics.resource_metrics[0].scope_metrics[0].metrics), 1
        )
        metric = metrics.resource_metrics[0].scope_metrics[0].metrics[0]
        self.assertEqual(metric.name, "testhistogram")
        self.assertEqual(
            metric.data.data_points[0].explicit_bounds, (1.0, 2.0, 3.0)
        )

    def test_view_overrides_buckets(self):
        reader = InMemoryMetricReader()
        view = View(
            instrument_name="testhistogram",
            aggregation=ExplicitBucketHistogramAggregation(
                boundaries=[10.0, 100.0, 1000.0]
            ),
        )
        meter_provider = MeterProvider(
            metric_readers=[reader],
            views=[view],
        )
        meter = meter_provider.get_meter("testmeter")
        histogram = meter.create_histogram(
            "testhistogram",
            explicit_bucket_boundaries_advisory=[1.0, 2.0, 3.0],
        )
        histogram.record(1, {"label": "value"})
        histogram.record(2, {"label": "value"})
        histogram.record(3, {"label": "value"})

        metrics = reader.get_metrics_data()
        self.assertEqual(len(metrics.resource_metrics), 1)
        self.assertEqual(len(metrics.resource_metrics[0].scope_metrics), 1)
        self.assertEqual(
            len(metrics.resource_metrics[0].scope_metrics[0].metrics), 1
        )
        metric = metrics.resource_metrics[0].scope_metrics[0].metrics[0]
        self.assertEqual(metric.name, "testhistogram")
        self.assertEqual(
            metric.data.data_points[0].explicit_bounds, (10.0, 100.0, 1000.0)
        )

    def test_explicit_aggregation(self):
        reader = InMemoryMetricReader(
            preferred_aggregation={
                Histogram: ExplicitBucketHistogramAggregation()
            }
        )
        meter_provider = MeterProvider(
            metric_readers=[reader],
        )
        meter = meter_provider.get_meter("testmeter")
        histogram = meter.create_histogram(
            "testhistogram",
            explicit_bucket_boundaries_advisory=[1.0, 2.0, 3.0],
        )
        histogram.record(1, {"label": "value"})
        histogram.record(2, {"label": "value"})
        histogram.record(3, {"label": "value"})

        metrics = reader.get_metrics_data()
        self.assertEqual(len(metrics.resource_metrics), 1)
        self.assertEqual(len(metrics.resource_metrics[0].scope_metrics), 1)
        self.assertEqual(
            len(metrics.resource_metrics[0].scope_metrics[0].metrics), 1
        )
        metric = metrics.resource_metrics[0].scope_metrics[0].metrics[0]
        self.assertEqual(metric.name, "testhistogram")
        self.assertEqual(
            metric.data.data_points[0].explicit_bounds, (1.0, 2.0, 3.0)
        )

    def test_explicit_aggregation_multiple_histograms(self):
        reader = InMemoryMetricReader(
            preferred_aggregation={
                Histogram: ExplicitBucketHistogramAggregation()
            }
        )
        meter_provider = MeterProvider(
            metric_readers=[reader],
        )
        meter = meter_provider.get_meter("testmeter")

        histogram1 = meter.create_histogram(
            "testhistogram1",
            explicit_bucket_boundaries_advisory=[1.0, 2.0, 3.0],
        )
        histogram1.record(1, {"label": "value"})
        histogram1.record(2, {"label": "value"})
        histogram1.record(3, {"label": "value"})

        histogram2 = meter.create_histogram(
            "testhistogram2",
            explicit_bucket_boundaries_advisory=[4.0, 5.0, 6.0],
        )
        histogram2.record(4, {"label": "value"})
        histogram2.record(5, {"label": "value"})
        histogram2.record(6, {"label": "value"})

        metrics = reader.get_metrics_data()
        self.assertEqual(len(metrics.resource_metrics), 1)
        self.assertEqual(len(metrics.resource_metrics[0].scope_metrics), 1)
        self.assertEqual(
            len(metrics.resource_metrics[0].scope_metrics[0].metrics), 2
        )
        metric1 = metrics.resource_metrics[0].scope_metrics[0].metrics[0]
        self.assertEqual(metric1.name, "testhistogram1")
        self.assertEqual(
            metric1.data.data_points[0].explicit_bounds, (1.0, 2.0, 3.0)
        )
        metric2 = metrics.resource_metrics[0].scope_metrics[0].metrics[1]
        self.assertEqual(metric2.name, "testhistogram2")
        self.assertEqual(
            metric2.data.data_points[0].explicit_bounds, (4.0, 5.0, 6.0)
        )

    def test_explicit_aggregation_default_boundaries(self):
        reader = InMemoryMetricReader(
            preferred_aggregation={
                Histogram: ExplicitBucketHistogramAggregation()
            }
        )
        meter_provider = MeterProvider(
            metric_readers=[reader],
        )
        meter = meter_provider.get_meter("testmeter")

        histogram = meter.create_histogram(
            "testhistogram",
        )
        histogram.record(1, {"label": "value"})
        histogram.record(2, {"label": "value"})
        histogram.record(3, {"label": "value"})

        metrics = reader.get_metrics_data()
        self.assertEqual(len(metrics.resource_metrics), 1)
        self.assertEqual(len(metrics.resource_metrics[0].scope_metrics), 1)
        self.assertEqual(
            len(metrics.resource_metrics[0].scope_metrics[0].metrics), 1
        )
        metric = metrics.resource_metrics[0].scope_metrics[0].metrics[0]
        self.assertEqual(metric.name, "testhistogram")
        self.assertEqual(
            metric.data.data_points[0].explicit_bounds,
            _DEFAULT_EXPLICIT_BUCKET_HISTOGRAM_AGGREGATION_BOUNDARIES,
        )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import unittest

from opentelemetry import trace as trace_api
from opentelemetry._logs import _internal as logging_api
from opentelemetry.metrics import _internal as metrics_api
from opentelemetry.metrics._internal import _ProxyMeterProvider
from opentelemetry.util._once import Once


# pylint: disable=protected-access
def reset_trace_globals() -> None:
    """WARNING: only use this for tests."""
    trace_api._TRACER_PROVIDER_SET_ONCE = Once()
    trace_api._TRACER_PROVIDER = None
    trace_api._PROXY_TRACER_PROVIDER = trace_api.ProxyTracerProvider()


# pylint: disable=protected-access
def reset_metrics_globals() -> None:
    """WARNING: only use this for tests."""
    metrics_api._METER_PROVIDER_SET_ONCE = Once()  # type: ignore[attr-defined]
    metrics_api._METER_PROVIDER = None  # type: ignore[attr-defined]
    metrics_api._PROXY_METER_PROVIDER = _ProxyMeterProvider()  # type: ignore[attr-defined]


# pylint: disable=protected-access
def reset_logging_globals() -> None:
    """WARNING: only use this for tests."""
    logging_api._LOGGER_PROVIDER_SET_ONCE = Once()  # type: ignore[attr-defined]
    logging_api._LOGGER_PROVIDER = None  # type: ignore[attr-defined]
    logging_api._PROXY_LOGGER_PROVIDER = logging_api.ProxyLoggerProvider()  # type: ignore[attr-defined]


# pylint: disable=protected-access
def reset_event_globals() -> None:
    """Deprecated no-op kept for backward compatibility.

    The Events API has been removed; this function does nothing.
    """


class TraceGlobalsTest(unittest.TestCase):
    """Resets trace API globals in setUp/tearDown

    Use as a base class or mixin for your test that modifies trace API globals.
    """

    def setUp(self) -> None:
        super().setUp()
        reset_trace_globals()

    def tearDown(self) -> None:
        super().tearDown()
        reset_trace_globals()


class MetricsGlobalsTest(unittest.TestCase):
    """Resets metrics API globals in setUp/tearDown

    Use as a base class or mixin for your test that modifies metrics API globals.
    """

    def setUp(self) -> None:
        super().setUp()
        reset_metrics_globals()

    def tearDown(self) -> None:
        super().tearDown()
        reset_metrics_globals()


class LoggingGlobalsTest(unittest.TestCase):
    """Resets logging API globals in setUp/tearDown

    Use as a base class or mixin for your test that modifies logging API globals.
    """

    def setUp(self) -> None:
        super().setUp()
        reset_logging_globals()

    def tearDown(self) -> None:
        super().tearDown()
        reset_logging_globals()


class EventsGlobalsTest(unittest.TestCase):
    """Deprecated no-op kept for backward compatibility.

    The Events API has been removed; this class does nothing.
    """

    def setUp(self) -> None:  # pylint: disable=useless-parent-delegation
        super().setUp()

    def tearDown(self) -> None:  # pylint: disable=useless-parent-delegation
        super().tearDown()



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from dataclasses import dataclass
from typing import TYPE_CHECKING

from opentelemetry.context import Context
from opentelemetry.util.types import Attributes

if TYPE_CHECKING:
    from opentelemetry.sdk.metrics._internal.instrument import _Instrument


@dataclass(frozen=True)
class Measurement:
    """
    Represents a data point reported via the metrics API to the SDK.

    Attributes:
        value: Measured value
        time_unix_nano: The time the API call was made to record the Measurement
        instrument: The instrument that produced this `Measurement`.
        context: The active Context of the Measurement at API call time.
        attributes: Measurement attributes
    """

    value: int | float
    time_unix_nano: int
    instrument: _Instrument
    context: Context
    attributes: Attributes = None



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import logging
import threading
import time


class RefCount:
    """Thread-safe counter"""

    def __init__(self, count=1):
        self._lock = threading.Lock()
        self._count = count

    def incr(self):
        with self._lock:
            self._count += 1
            return self._count

    def decr(self):
        with self._lock:
            self._count -= 1
            return self._count


def await_until(func, timeout=5.0):
    """Polls for func() to return True"""
    end_time = time.time() + timeout
    while time.time() < end_time and not func():
        time.sleep(0.01)


def stop_loop_when(loop, cond_func, timeout=5.0):
    """
    Registers a periodic callback that stops the loop when cond_func() == True.
    Compatible with both Tornado and asyncio.
    """
    if cond_func() or timeout <= 0.0:
        loop.stop()
        return

    timeout -= 0.1
    loop.call_later(0.1, stop_loop_when, loop, cond_func, timeout)


def get_logger(name):
    """Returns a logger with log level set to INFO"""
    logging.basicConfig(level=logging.INFO)
    return logging.getLogger(name)


def get_one_by_tag(spans, key, value):
    """Return a single Span with a tag value/key from a list,
    errors if more than one is found."""

    found = []
    for span in spans:
        if span.attributes.get(key) == value:
            found.append(span)

    if len(found) > 1:
        raise RuntimeError("Too many values")

    return found[0] if len(found) > 0 else None


def get_one_by_operation_name(spans, name):
    """Return a single Span with a name from a list,
    errors if more than one is found."""
    found = []
    for span in spans:
        if span.name == name:
            found.append(span)

    if len(found) > 1:
        raise RuntimeError("Too many values")

    return found[0] if len(found) > 0 else None



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import collections
import enum
import inspect
import logging
import os
import threading
import time
import weakref
from abc import abstractmethod
from typing import (
    Generic,
    Protocol,
    TypeVar,
)

from opentelemetry.context import (
    _SUPPRESS_INSTRUMENTATION_KEY,
    attach,
    detach,
    set_value,
)
from opentelemetry.sdk._shared_internal._processor_metrics import (
    ProcessorMetricsT,
)
from opentelemetry.util._once import Once


class DuplicateFilter(logging.Filter):
    """Filter that can be applied to internal `logger`'s.

    Currently applied to `logger`s on the export logs path to prevent endlessly logging the same log
    in cases where logging itself is failing."""

    def filter(self, record):
        current_log = (
            record.module,
            record.levelno,
            record.msg,
            # We need to pick a time longer than the OTLP LogExporter timeout
            # which defaults to 10 seconds, but not pick something so long that
            # it filters out useful logs.
            time.time() // 20,
        )
        if current_log != getattr(self, "last_log", None):
            self.last_log = current_log  # pylint: disable=attribute-defined-outside-init
            return True
        # False means python's `logging` module will no longer process this log.
        return False


class BatchExportStrategy(enum.Enum):
    EXPORT_ALL = 0
    EXPORT_WHILE_BATCH_EXCEEDS_THRESHOLD = 1
    EXPORT_AT_LEAST_ONE_BATCH = 2


Telemetry = TypeVar("Telemetry")


class Exporter(Protocol[Telemetry]):
    @abstractmethod
    def export(self, batch: list[Telemetry], /):
        raise NotImplementedError

    @abstractmethod
    def shutdown(self):
        raise NotImplementedError


_logger = logging.getLogger(__name__)
_logger.addFilter(DuplicateFilter())


class BatchProcessor(Generic[Telemetry]):
    """This class can be used with exporter's that implement the above
    Exporter interface to buffer and send telemetry in batch through
     the exporter."""

    def __init__(
        self,
        exporter: Exporter[Telemetry],
        schedule_delay_millis: float,
        max_export_batch_size: int,
        export_timeout_millis: float,
        max_queue_size: int,
        exporting: str,
        metrics: ProcessorMetricsT,
    ):
        self._bsp_reset_once = Once()
        self._exporter = exporter
        self._max_queue_size = max_queue_size
        self._schedule_delay_millis = schedule_delay_millis
        self._schedule_delay = schedule_delay_millis / 1e3
        self._max_export_batch_size = max_export_batch_size
        # Not used. No way currently to pass timeout to export.
        # TODO(https://github.com/open-telemetry/opentelemetry-python/issues/4555): figure out what this should do.
        self._export_timeout_millis = export_timeout_millis
        # Deque is thread safe.
        self._queue = collections.deque([], max_queue_size)
        self._worker_thread = threading.Thread(
            name=f"OtelBatch{exporting}RecordProcessor",
            target=self.worker,
            daemon=True,
        )
        self._exporting = exporting

        self._shutdown = False
        self._shutdown_timeout_exceeded = False
        self._export_lock = threading.Lock()
        self._worker_awaken = threading.Event()
        self._worker_thread.start()
        if hasattr(os, "register_at_fork"):
            weak_reinit = weakref.WeakMethod(self._at_fork_reinit)
            os.register_at_fork(after_in_child=lambda: weak_reinit()())  # pyright: ignore[reportOptionalCall] pylint: disable=unnecessary-lambda
        self._pid = os.getpid()

        metrics.register_queue_size(lambda: len(self._queue))
        self._metrics = metrics

    def _should_export_batch(
        self, batch_strategy: BatchExportStrategy, num_iterations: int
    ) -> bool:
        if not self._queue or self._shutdown_timeout_exceeded:
            return False
        # Always continue to export while queue length exceeds max batch size.
        if len(self._queue) >= self._max_export_batch_size:
            return True
        if batch_strategy is BatchExportStrategy.EXPORT_ALL:
            return True
        if batch_strategy is BatchExportStrategy.EXPORT_AT_LEAST_ONE_BATCH:
            return num_iterations == 0
        return False

    def _at_fork_reinit(self):
        self._export_lock = threading.Lock()
        self._worker_awaken = threading.Event()
        self._queue.clear()
        self._worker_thread = threading.Thread(
            name=f"OtelBatch{self._exporting}RecordProcessor",
            target=self.worker,
            daemon=True,
        )
        self._worker_thread.start()
        self._pid = os.getpid()

    def worker(self):
        while not self._shutdown:
            # Lots of strategies in the spec for setting next timeout.
            # https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/trace/sdk.md#batching-processor.
            # Shutdown will interrupt this sleep. Emit will interrupt this sleep only if the queue is bigger then threshold.
            sleep_interrupted = self._worker_awaken.wait(self._schedule_delay)
            if self._shutdown:
                break
            self._export(
                BatchExportStrategy.EXPORT_WHILE_BATCH_EXCEEDS_THRESHOLD
                if sleep_interrupted
                else BatchExportStrategy.EXPORT_AT_LEAST_ONE_BATCH
            )
            self._worker_awaken.clear()
        self._export(BatchExportStrategy.EXPORT_ALL)

    def _export(self, batch_strategy: BatchExportStrategy) -> None:
        with self._export_lock:
            iteration = 0
            # We could see concurrent export calls from worker and force_flush. We call _should_export_batch
            # once the lock is obtained to see if we still need to make the requested export.
            while self._should_export_batch(batch_strategy, iteration):
                iteration += 1
                token = attach(set_value(_SUPPRESS_INSTRUMENTATION_KEY, True))
                error: Exception | None = None
                count = 0
                try:
                    count = min(
                        self._max_export_batch_size,
                        len(self._queue),
                    )
                    self._exporter.export(
                        [
                            # Oldest records are at the back, so pop from there.
                            self._queue.pop()
                            for _ in range(count)
                        ]
                    )
                except Exception as err:  # pylint: disable=broad-exception-caught
                    error = err
                    _logger.exception(
                        "Exception while exporting %s.", self._exporting
                    )
                finally:
                    self._metrics.finish_items(count, error)
                detach(token)

    def emit(self, data: Telemetry) -> None:
        if self._shutdown:
            _logger.info("Shutdown called, ignoring %s.", self._exporting)
            return
        if self._pid != os.getpid():
            self._bsp_reset_once.do_once(self._at_fork_reinit)
        if len(self._queue) == self._max_queue_size:
            _logger.warning("Queue full, dropping %s.", self._exporting)
            self._metrics.drop_items(1)
        # This will drop a log from the right side if the queue is at _max_queue_size.
        self._queue.appendleft(data)
        if (
            len(self._queue) >= self._max_export_batch_size
            and not self._worker_awaken.is_set()
        ):
            self._worker_awaken.set()

    def shutdown(self, timeout_millis: int = 30000):
        if self._shutdown:
            return
        shutdown_should_end = time.time() + (timeout_millis / 1000)
        # Causes emit to reject telemetry and makes force_flush a no-op.
        self._shutdown = True
        # Interrupts sleep in the worker if it's sleeping.
        self._worker_awaken.set()
        self._worker_thread.join(timeout_millis / 1000)
        # Stops worker thread from calling export again if queue is still not empty.
        self._shutdown_timeout_exceeded = True
        # We want to shutdown immediately only if we already waited `timeout_secs`.
        # Otherwise we pass the remaining timeout to the exporter.
        # Some exporter's shutdown support a timeout param.
        if (
            "timeout_millis"
            in inspect.getfullargspec(self._exporter.shutdown).args
        ):
            remaining_millis = (shutdown_should_end - time.time()) * 1000
            self._exporter.shutdown(timeout_millis=max(0, remaining_millis))  # type: ignore
        else:
            self._exporter.shutdown()
        # Worker thread **should** be finished at this point, because we called shutdown on the exporter,
        # and set shutdown_is_occuring to prevent further export calls. It's possible that a single export
        # call is ongoing and the thread isn't finished. In this case we will return instead of waiting on
        # the thread to finish.

    # TODO: Fix force flush so the timeout is used https://github.com/open-telemetry/opentelemetry-python/issues/4568.
    def force_flush(self, timeout_millis: int | None = None) -> bool:
        if self._shutdown:
            return False
        # Blocking call to export.
        self._export(BatchExportStrategy.EXPORT_ALL)
        return True



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# pylint: disable=protected-access,invalid-name,no-self-use

import gc
import math
import weakref
from logging import WARNING
from time import sleep, time_ns
from typing import cast
from unittest.mock import Mock, patch

import pytest

from opentelemetry.sdk.environment_variables import (
    OTEL_PYTHON_SDK_INTERNAL_METRICS_ENABLED,
)
from opentelemetry.sdk.metrics import (
    Counter,
    MeterProvider,
    MetricsTimeoutError,
)
from opentelemetry.sdk.metrics._internal import _Counter
from opentelemetry.sdk.metrics._internal.point import (
    HistogramDataPoint,
    MetricsData,
    ResourceMetrics,
    ScopeMetrics,
)
from opentelemetry.sdk.metrics.export import (
    AggregationTemporality,
    Gauge,
    Metric,
    MetricExporter,
    MetricExportResult,
    NumberDataPoint,
    PeriodicExportingMetricReader,
    Sum,
)
from opentelemetry.sdk.metrics.view import (
    DefaultAggregation,
    LastValueAggregation,
)
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.util.instrumentation import InstrumentationScope
from opentelemetry.test.concurrency_test import ConcurrencyTestBase


class FakeMetricsExporter(MetricExporter):
    def __init__(
        self, wait=0, preferred_temporality=None, preferred_aggregation=None
    ):
        self.wait = wait
        self.metrics: list[MetricsData] = []
        self._shutdown = False
        super().__init__(
            preferred_temporality=preferred_temporality,
            preferred_aggregation=preferred_aggregation,
        )

    def export(
        self,
        metrics_data: MetricsData,
        timeout_millis: float = 10_000,
        **kwargs,
    ) -> MetricExportResult:
        sleep(self.wait)
        self.metrics.append(metrics_data)
        return MetricExportResult.SUCCESS

    def shutdown(self, timeout_millis: float = 30_000, **kwargs) -> None:
        self._shutdown = True

    def force_flush(self, timeout_millis: float = 10_000) -> bool:
        return True


class ExceptionAtCollectionPeriodicExportingMetricReader(
    PeriodicExportingMetricReader
):
    def __init__(
        self,
        exporter: MetricExporter,
        exception: Exception,
        export_interval_millis: float | None = None,
        export_timeout_millis: float | None = None,
    ) -> None:
        super().__init__(
            exporter, export_interval_millis, export_timeout_millis
        )
        self._collect_exception = exception

    # pylint: disable=overridden-final-method
    def collect(self, timeout_millis: float = 10_000) -> None:
        raise self._collect_exception


metrics_list = [
    Metric(
        name="sum_name",
        description="",
        unit="",
        data=Sum(
            data_points=[
                NumberDataPoint(
                    attributes={},
                    start_time_unix_nano=time_ns(),
                    time_unix_nano=time_ns(),
                    value=2,
                )
            ],
            aggregation_temporality=1,
            is_monotonic=True,
        ),
    ),
    Metric(
        name="gauge_name",
        description="",
        unit="",
        data=Gauge(
            data_points=[
                NumberDataPoint(
                    attributes={},
                    start_time_unix_nano=time_ns(),
                    time_unix_nano=time_ns(),
                    value=2,
                )
            ]
        ),
    ),
]
metrics = MetricsData(
    resource_metrics=[
        ResourceMetrics(
            scope_metrics=[
                ScopeMetrics(
                    metrics=metrics_list,
                    scope=InstrumentationScope(name="test"),
                    schema_url="",
                )
            ],
            resource=Resource.create(),
            schema_url="",
        )
    ]
)


class TestPeriodicExportingMetricReader(ConcurrencyTestBase):
    def test_defaults(self):
        pmr = PeriodicExportingMetricReader(FakeMetricsExporter())
        self.assertEqual(pmr._export_interval_millis, 60000)
        self.assertEqual(pmr._export_timeout_millis, 30000)
        with self.assertLogs(level=WARNING):
            pmr.shutdown()

    def _create_periodic_reader(
        self,
        metrics_data: MetricsData,
        exporter,
        collect_wait=0,
        interval=60000,
        timeout=30000,
    ):
        pmr = PeriodicExportingMetricReader(
            exporter,
            export_interval_millis=interval,
            export_timeout_millis=timeout,
        )

        def _collect(reader, timeout_millis):
            sleep(collect_wait)
            return metrics_data

        pmr._set_collect_callback(_collect)
        return pmr

    def test_ticker_called(self):
        collect_mock = Mock()
        exporter = FakeMetricsExporter()
        exporter.export = Mock()
        pmr = PeriodicExportingMetricReader(exporter, export_interval_millis=1)
        pmr._set_collect_callback(collect_mock)
        sleep(0.1)
        self.assertTrue(collect_mock.assert_called_once)
        pmr.shutdown()

    def test_ticker_not_called_on_infinity(self):
        collect_mock = Mock()
        exporter = FakeMetricsExporter()
        exporter.export = Mock()
        pmr = PeriodicExportingMetricReader(
            exporter, export_interval_millis=math.inf
        )
        pmr._set_collect_callback(collect_mock)
        sleep(0.1)
        self.assertTrue(collect_mock.assert_not_called)
        pmr.shutdown()

    def test_ticker_value_exception_on_zero(self):
        exporter = FakeMetricsExporter()
        exporter.export = Mock()
        self.assertRaises(
            ValueError,
            PeriodicExportingMetricReader,
            exporter,
            export_interval_millis=0,
        )

    def test_ticker_value_exception_on_negative(self):
        exporter = FakeMetricsExporter()
        exporter.export = Mock()
        self.assertRaises(
            ValueError,
            PeriodicExportingMetricReader,
            exporter,
            export_interval_millis=-100,
        )

    @pytest.mark.flaky(max_runs=3, min_passes=1)
    def test_ticker_collects_metrics(self):
        exporter = FakeMetricsExporter()

        pmr = self._create_periodic_reader(metrics, exporter, interval=100)
        sleep(0.15)
        self.assertEqual(exporter.metrics[0], metrics)
        pmr.shutdown()

    def test_shutdown(self):
        exporter = FakeMetricsExporter()

        pmr = self._create_periodic_reader(
            MetricsData(resource_metrics=[]), exporter
        )
        pmr.shutdown()
        self.assertEqual(exporter.metrics[0], MetricsData(resource_metrics=[]))
        self.assertTrue(pmr._shutdown)
        self.assertTrue(exporter._shutdown)

    def test_shutdown_multiple_times(self):
        pmr = self._create_periodic_reader(
            MetricsData(resource_metrics=[]), FakeMetricsExporter()
        )
        with self.assertLogs(level="WARNING") as w:
            self.run_with_many_threads(pmr.shutdown)
        self.assertTrue("Can't shutdown multiple times" in w.output[0])
        with self.assertLogs(level="WARNING") as w:
            pmr.shutdown()

    def test_exporter_temporality_preference(self):
        exporter = FakeMetricsExporter(
            preferred_temporality={
                Counter: AggregationTemporality.DELTA,
            },
        )
        pmr = PeriodicExportingMetricReader(exporter)
        for key, value in pmr._instrument_class_temporality.items():
            if key is not _Counter:
                self.assertEqual(value, AggregationTemporality.CUMULATIVE)
            else:
                self.assertEqual(value, AggregationTemporality.DELTA)

    def test_exporter_aggregation_preference(self):
        exporter = FakeMetricsExporter(
            preferred_aggregation={
                Counter: LastValueAggregation(),
            },
        )
        pmr = PeriodicExportingMetricReader(exporter)
        for key, value in pmr._instrument_class_aggregation.items():
            if key is not _Counter:
                self.assertTrue(isinstance(value, DefaultAggregation))
            else:
                self.assertTrue(isinstance(value, LastValueAggregation))

    def test_metric_timeout_does_not_kill_worker_thread(self):
        exporter = FakeMetricsExporter()
        pmr = ExceptionAtCollectionPeriodicExportingMetricReader(
            exporter,
            MetricsTimeoutError("test timeout"),
            export_timeout_millis=1,
        )

        sleep(0.1)
        self.assertTrue(pmr._daemon_thread.is_alive())
        pmr.shutdown()

    def test_metric_exporer_gc(self):
        # Given a PeriodicExportingMetricReader
        exporter = FakeMetricsExporter(
            preferred_aggregation={
                Counter: LastValueAggregation(),
            },
        )
        processor = PeriodicExportingMetricReader(exporter)
        weak_ref = weakref.ref(processor)
        processor.shutdown()

        # When we garbage collect the reader
        del processor
        gc.collect()

        # Then the reference to the reader should no longer exist
        self.assertIsNone(
            weak_ref(),
            "The PeriodicExportingMetricReader object created by this test wasn't garbage collected",
        )

    @patch.dict(
        "os.environ", {OTEL_PYTHON_SDK_INTERNAL_METRICS_ENABLED: "true"}
    )
    def test_metric_reader_metrics(self):
        exporter = FakeMetricsExporter()
        pmr = PeriodicExportingMetricReader(
            exporter, export_interval_millis=100000
        )
        mp = MeterProvider(metric_readers=[pmr])

        counter = mp.get_meter("test").create_counter("test_counter")
        counter.add(1)

        mp.force_flush()
        self.assertEqual(len(exporter.metrics), 1)
        # Need a second collection to get the metric we recorded during first collection
        exporter.metrics.clear()
        mp.force_flush()
        self.assertEqual(len(exporter.metrics), 1)
        metric_data = exporter.metrics[0]

        scope_metrics = [
            sm
            for sm in metric_data.resource_metrics[0].scope_metrics
            if sm.scope.name == "opentelemetry-sdk"
        ]
        self.assertEqual(len(scope_metrics), 1)
        reader_metrics = [
            m
            for m in scope_metrics[0].metrics
            if m.name == "otel.sdk.metric_reader.collection.duration"
        ]
        self.assertEqual(len(reader_metrics), 1)
        metric = reader_metrics[0]

        point = metric.data.data_points[0]
        histogram = cast(HistogramDataPoint, point)
        self.assertEqual(histogram.count, 1)
        attrs = histogram.attributes
        assert attrs is not None
        self.assertEqual(
            attrs["otel.component.type"], "periodic_metric_reader"
        )
        name = attrs["otel.component.name"]
        assert isinstance(name, str)
        self.assertTrue(name.startswith("periodic_metric_reader/"))

        mp.shutdown()



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

"""Zipkin Exporter Endpoints"""

import ipaddress

from opentelemetry import trace
from opentelemetry.sdk.resources import SERVICE_NAME, Resource

IpInput = str | int | None


class NodeEndpoint:
    """The network context of a node in the service graph.

    Args:
        ipv4: Primary IPv4 address associated with this connection.
        ipv6: Primary IPv6 address associated with this connection.
        port: Depending on context, this could be a listen port or the
          client-side of a socket. None if unknown.
    """

    def __init__(
        self,
        ipv4: IpInput = None,
        ipv6: IpInput = None,
        port: int | None = None,
    ):
        self.ipv4 = ipv4
        self.ipv6 = ipv6
        self.port = port

        tracer_provider = trace.get_tracer_provider()

        if hasattr(tracer_provider, "resource"):
            resource = tracer_provider.resource
        else:
            resource = Resource.create()

        self.service_name = resource.attributes[SERVICE_NAME]

    @property
    def ipv4(self) -> ipaddress.IPv4Address | None:
        return self._ipv4

    @ipv4.setter
    def ipv4(self, address: IpInput) -> None:
        if address is None:
            self._ipv4 = None
        else:
            ipv4_address = ipaddress.ip_address(address)
            if not isinstance(ipv4_address, ipaddress.IPv4Address):
                raise ValueError(
                    f"{address!r} does not appear to be an IPv4 address"
                )
            self._ipv4 = ipv4_address

    @property
    def ipv6(self) -> ipaddress.IPv6Address | None:
        return self._ipv6

    @ipv6.setter
    def ipv6(self, address: IpInput) -> None:
        if address is None:
            self._ipv6 = None
        else:
            ipv6_address = ipaddress.ip_address(address)
            if not isinstance(ipv6_address, ipaddress.IPv6Address):
                raise ValueError(
                    f"{address!r} does not appear to be an IPv6 address"
                )
            self._ipv6 = ipv6_address



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

import unittest
import unittest.mock

import requests

from opentelemetry._logs import SeverityNumber
from opentelemetry.sdk._logs import LoggerProvider
from opentelemetry.sdk._logs.export import SimpleLogRecordProcessor
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import PeriodicExportingMetricReader
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import SimpleSpanProcessor
from opentelemetry.test._otlp_test_server import (
    OtlpProtoTestServer,
    RecordedLogRecord,
    RecordedMetric,
    RecordedSpan,
)

try:
    # pylint: disable-next=unused-import
    import opentelemetry.proto  # noqa: F401
    from opentelemetry.exporter.otlp.proto.http import Compression
    from opentelemetry.exporter.otlp.proto.http._log_exporter import (
        OTLPLogExporter,
    )
    from opentelemetry.exporter.otlp.proto.http.metric_exporter import (
        OTLPMetricExporter,
    )
    from opentelemetry.exporter.otlp.proto.http.trace_exporter import (
        OTLPSpanExporter,
    )

    _HAS_PROTO_DEPS = True
except ImportError:
    _HAS_PROTO_DEPS = False


@unittest.skipUnless(
    _HAS_PROTO_DEPS,
    "opentelemetry-exporter-otlp-proto-http or opentelemetry-proto are not installed",
)
class TestOtlpProtoTestServer(unittest.TestCase):
    def setUp(self):
        self.server = OtlpProtoTestServer()
        self.server.start()

    def tearDown(self):
        self.server.stop()

    def _make_trace_provider(
        self, service: str = "trace-svc", **exporter_kwargs
    ) -> TracerProvider:
        provider = TracerProvider(
            resource=Resource.create({"service.name": service})
        )
        provider.add_span_processor(
            SimpleSpanProcessor(
                OTLPSpanExporter(
                    endpoint=self.server.traces_endpoint,
                    timeout=1,
                    **exporter_kwargs,
                )
            )
        )
        return provider

    def _make_metrics_provider(
        self, service: str = "metrics-svc", **exporter_kwargs
    ) -> tuple[PeriodicExportingMetricReader, MeterProvider]:
        reader = PeriodicExportingMetricReader(
            OTLPMetricExporter(
                endpoint=self.server.metrics_endpoint,
                timeout=1,
                **exporter_kwargs,
            ),
            export_interval_millis=100,
        )
        provider = MeterProvider(
            resource=Resource.create({"service.name": service}),
            metric_readers=[reader],
        )
        return reader, provider

    def _make_log_provider(
        self, service: str = "log-svc", **exporter_kwargs
    ) -> LoggerProvider:
        provider = LoggerProvider(
            resource=Resource.create({"service.name": service})
        )
        provider.add_log_record_processor(
            SimpleLogRecordProcessor(
                OTLPLogExporter(
                    endpoint=self.server.logs_endpoint,
                    timeout=1,
                    **exporter_kwargs,
                )
            )
        )
        return provider

    def test_span_export(self):
        provider = self._make_trace_provider()
        tracer = provider.get_tracer("trace-scope")
        with tracer.start_as_current_span("foo"):
            with tracer.start_as_current_span("bar"):
                pass
        tracer.start_span("baz").end()

        spans = self.server.get_spans(count=3, timeout=5.0)
        self.assertEqual(
            {recorded.span.name for recorded in spans}, {"foo", "bar", "baz"}
        )
        for recorded in spans:
            self.assertIsInstance(recorded, RecordedSpan)
            self.assertEqual(recorded.scope.name, "trace-scope")
            svc = next(
                a.value.string_value
                for a in recorded.resource.attributes
                if a.key == "service.name"
            )
            self.assertEqual(svc, "trace-svc")
        provider.shutdown()

    def test_wait_for_span(self):
        provider = self._make_trace_provider()
        tracer = provider.get_tracer("trace-scope")
        tracer.start_span("alpha").end()
        tracer.start_span("beta").end()

        found = self.server.wait_for_span(name="beta", timeout=5.0)
        self.assertEqual(found.span.name, "beta")
        with self.assertRaises(TimeoutError):
            self.server.get_span(timeout=0.1)
        provider.shutdown()

    def test_span_drain_and_clear(self):
        provider = self._make_trace_provider()
        tracer = provider.get_tracer("trace-scope")

        tracer.start_span("d1").end()
        tracer.start_span("d2").end()
        drained = self.server.drain_spans()
        self.assertEqual({r.span.name for r in drained}, {"d1", "d2"})

        tracer.start_span("c1").end()
        tracer.start_span("c2").end()
        self.server.clear()
        with self.assertRaises(TimeoutError):
            self.server.get_span(timeout=0.1)
        provider.shutdown()

    def test_metric_export(self):
        reader, provider = self._make_metrics_provider()
        meter = provider.get_meter("metrics-scope")
        meter.create_counter("test.requests").add(5, {"env": "test"})
        meter.create_histogram("test.latency").record(1.5, {"route": "/api"})
        reader.force_flush(timeout_millis=3000)

        metrics = self.server.get_metrics(count=2, timeout=5.0)
        self.assertEqual(
            {r.metric.name for r in metrics}, {"test.requests", "test.latency"}
        )
        for recorded in metrics:
            self.assertIsInstance(recorded, RecordedMetric)
            self.assertEqual(recorded.scope.name, "metrics-scope")
            svc = next(
                a.value.string_value
                for a in recorded.resource.attributes
                if a.key == "service.name"
            )
            self.assertEqual(svc, "metrics-svc")
        provider.shutdown()

    def test_wait_for_metric(self):
        reader, provider = self._make_metrics_provider()
        meter = provider.get_meter("metrics-scope")
        meter.create_counter("other.counter").add(1)
        meter.create_counter("target.counter").add(1)
        reader.force_flush(timeout_millis=3000)

        found = self.server.wait_for_metric(name="target.counter", timeout=5.0)
        self.assertEqual(found.metric.name, "target.counter")
        provider.shutdown()

    def test_metric_drain_and_timeout(self):
        reader, provider = self._make_metrics_provider()
        meter = provider.get_meter("metrics-scope")
        meter.create_counter("drain.counter").add(1)
        reader.force_flush(timeout_millis=3000)
        self.server.get_metric(timeout=5.0)

        self.assertEqual(self.server.drain_metrics(), [])
        provider.shutdown()
        self.server.drain_metrics()
        with self.assertRaises(TimeoutError):
            self.server.get_metric(timeout=0.1)

    def test_log_export(self):
        provider = self._make_log_provider()
        logger = provider.get_logger("log-scope")
        logger.emit(body="first message", severity_number=SeverityNumber.WARN)
        logger.emit(
            body="second message", severity_number=SeverityNumber.ERROR
        )

        log_records = self.server.get_log_records(count=2, timeout=5.0)
        self.assertEqual(
            {r.log_record.body.string_value for r in log_records},
            {"first message", "second message"},
        )
        for recorded in log_records:
            self.assertIsInstance(recorded, RecordedLogRecord)
            self.assertEqual(recorded.scope.name, "log-scope")
            svc = next(
                a.value.string_value
                for a in recorded.resource.attributes
                if a.key == "service.name"
            )
            self.assertEqual(svc, "log-svc")
        provider.shutdown()

    def test_wait_for_log_record(self):
        provider = self._make_log_provider()
        logger = provider.get_logger("log-scope")
        logger.emit(body="wait-target", severity_number=SeverityNumber.INFO)

        found = self.server.wait_for_log_record(timeout=5.0)
        self.assertIsInstance(found, RecordedLogRecord)
        self.assertEqual(found.log_record.body.string_value, "wait-target")
        provider.shutdown()

    def test_log_drain_and_timeout(self):
        provider = self._make_log_provider()
        logger = provider.get_logger("log-scope")
        logger.emit(body="drain-me", severity_number=SeverityNumber.DEBUG)
        self.server.get_log_record(timeout=5.0)

        self.assertEqual(self.server.drain_log_records(), [])
        with self.assertRaises(TimeoutError):
            self.server.get_log_record(timeout=0.1)
        provider.shutdown()

    def test_compression(self):
        trace_provider = self._make_trace_provider(
            compression=Compression.Gzip
        )
        metrics_reader, metrics_provider = self._make_metrics_provider(
            compression=Compression.Gzip
        )
        log_provider = self._make_log_provider(compression=Compression.Gzip)

        trace_provider.get_tracer("s").start_span("gzip-span").end()
        self.assertEqual(
            self.server.get_span(timeout=5.0).span.name, "gzip-span"
        )

        metrics_provider.get_meter("s").create_counter("gzip.counter").add(1)
        metrics_reader.force_flush(timeout_millis=3000)
        self.assertEqual(
            self.server.get_metric(timeout=5.0).metric.name, "gzip.counter"
        )

        log_provider.get_logger("s").emit(
            body="gzip log", severity_number=SeverityNumber.INFO
        )
        self.assertEqual(
            self.server.get_log_record(
                timeout=5.0
            ).log_record.body.string_value,
            "gzip log",
        )

        trace_provider.shutdown()
        metrics_provider.shutdown()
        log_provider.shutdown()

    def test_endpoint_urls(self):
        port = self.server.port
        self.assertGreater(port, 0)
        self.assertEqual(
            self.server.traces_endpoint, f"http://127.0.0.1:{port}/v1/traces"
        )
        self.assertEqual(
            self.server.metrics_endpoint, f"http://127.0.0.1:{port}/v1/metrics"
        )
        self.assertEqual(
            self.server.logs_endpoint, f"http://127.0.0.1:{port}/v1/logs"
        )

    def test_context_manager(self):
        with OtlpProtoTestServer() as srv:
            provider = TracerProvider()
            provider.add_span_processor(
                SimpleSpanProcessor(
                    OTLPSpanExporter(endpoint=srv.traces_endpoint, timeout=1)
                )
            )
            provider.get_tracer("s").start_span("ctx-span").end()
            self.assertEqual(srv.get_span(timeout=5.0).span.name, "ctx-span")
            provider.shutdown()

    def test_base_path(self):
        with OtlpProtoTestServer(base_path="/custom") as srv:
            self.assertTrue(srv.traces_endpoint.endswith("/custom/v1/traces"))
            provider = TracerProvider()
            provider.add_span_processor(
                SimpleSpanProcessor(
                    OTLPSpanExporter(endpoint=srv.traces_endpoint, timeout=1)
                )
            )
            provider.get_tracer("s").start_span("prefixed-span").end()
            self.assertEqual(
                srv.get_span(timeout=5.0).span.name, "prefixed-span"
            )
            provider.shutdown()

    def test_signal_routing(self):
        trace_provider = self._make_trace_provider()
        metrics_reader, metrics_provider = self._make_metrics_provider()
        log_provider = self._make_log_provider()

        trace_provider.get_tracer("s").start_span("routed-span").end()
        metrics_provider.get_meter("s").create_counter("routed.counter").add(1)
        metrics_reader.force_flush(timeout_millis=3000)
        log_provider.get_logger("s").emit(
            body="routed log", severity_number=SeverityNumber.INFO
        )

        self.assertEqual(
            self.server.get_span(timeout=5.0).span.name, "routed-span"
        )
        self.assertEqual(
            self.server.get_metric(timeout=5.0).metric.name, "routed.counter"
        )
        self.assertEqual(
            self.server.get_log_record(
                timeout=5.0
            ).log_record.body.string_value,
            "routed log",
        )

        trace_provider.shutdown()
        metrics_provider.shutdown()
        log_provider.shutdown()

    def test_unknown_path_returns_404(self):
        resp = requests.post(
            f"http://127.0.0.1:{self.server.port}/unknown",
            data=b"",
            headers={"Content-Type": "application/x-protobuf"},
            timeout=2,
        )
        self.assertEqual(resp.status_code, 404)

    def test_missing_proto_raises_import_error(self):
        with unittest.mock.patch.dict(
            "sys.modules", {"opentelemetry.proto": None}
        ):
            with self.assertRaises(ImportError) as cm:
                OtlpProtoTestServer()
        self.assertIn("opentelemetry-proto", str(cm.exception))



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# metrics.py
# This is still work in progress as the metrics SDK is being implemented

from collections.abc import Iterable

from opentelemetry.metrics import (
    CallbackOptions,
    Observation,
    get_meter_provider,
    set_meter_provider,
)
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import (
    ConsoleMetricExporter,
    PeriodicExportingMetricReader,
)

exporter = ConsoleMetricExporter()
reader = PeriodicExportingMetricReader(exporter)
provider = MeterProvider(metric_readers=[reader])
set_meter_provider(provider)


def observable_counter_func(options: CallbackOptions) -> Iterable[Observation]:
    yield Observation(1, {})


def observable_up_down_counter_func(
    options: CallbackOptions,
) -> Iterable[Observation]:
    yield Observation(-10, {})


def observable_gauge_func(options: CallbackOptions) -> Iterable[Observation]:
    yield Observation(9, {})


meter = get_meter_provider().get_meter("getting-started", "0.1.2")

# Counter
counter = meter.create_counter("counter")
counter.add(1)

# Async Counter
observable_counter = meter.create_observable_counter(
    "observable_counter", [observable_counter_func]
)

# UpDownCounter
updown_counter = meter.create_up_down_counter("updown_counter")
updown_counter.add(1)
updown_counter.add(-5)

# Async UpDownCounter
observable_updown_counter = meter.create_observable_up_down_counter(
    "observable_updown_counter", [observable_up_down_counter_func]
)

# Histogram
histogram = meter.create_histogram("histogram")
histogram.record(99.9)

# Async Gauge
observable_gauge = meter.create_observable_gauge(
    "observable_gauge", [observable_gauge_func]
)

# Sync Gauge
gauge = meter.create_gauge("gauge")
gauge.set(1)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

"""
This library allows export of metrics data to `Prometheus <https://prometheus.io/>`_.

Usage
-----

The **OpenTelemetry Prometheus Exporter** allows export of `OpenTelemetry`_
metrics to `Prometheus`_.


.. _Prometheus: https://prometheus.io/
.. _OpenTelemetry: https://github.com/open-telemetry/opentelemetry-python/

.. code:: python

    from prometheus_client import start_http_server

    from opentelemetry.exporter.prometheus import PrometheusMetricReader
    from opentelemetry.metrics import get_meter_provider, set_meter_provider
    from opentelemetry.sdk.metrics import MeterProvider

    # Start Prometheus client
    start_http_server(port=8000, addr="localhost")

    # Exporter to export metrics to Prometheus
    prefix = "MyAppPrefix"
    reader = PrometheusMetricReader(prefix=prefix)

    # Meter is responsible for creating and recording metrics
    set_meter_provider(MeterProvider(metric_readers=[reader]))
    meter = get_meter_provider().get_meter("myapp", "0.1.2")

    counter = meter.create_counter(
        "requests",
        "requests",
        "number of requests",
    )

    # Labels are used to identify key-values that are associated with a specific
    # metric that you want to record. These are useful for pre-aggregation and can
    # be used to store custom dimensions pertaining to a metric
    labels = {"environment": "staging"}

    counter.add(25, labels)
    input("Press any key to exit...")

API
---
"""

from collections import deque
from collections.abc import Callable, Iterable, Sequence
from itertools import chain
from json import dumps
from logging import getLogger
from os import environ
from typing import (
    Any,
    TypeVar,
)

from prometheus_client import CollectorRegistry, start_http_server
from prometheus_client.core import (
    REGISTRY,
    CounterMetricFamily,
    GaugeMetricFamily,
    HistogramMetricFamily,
    InfoMetricFamily,
)
from prometheus_client.core import Metric as PrometheusMetric

from opentelemetry.exporter.prometheus._mapping import (
    map_unit,
    sanitize_attribute,
    sanitize_full_name,
)
from opentelemetry.sdk.environment_variables import (
    OTEL_EXPORTER_PROMETHEUS_HOST,
    OTEL_EXPORTER_PROMETHEUS_PORT,
)
from opentelemetry.sdk.metrics import (
    Counter,
    ObservableCounter,
    ObservableGauge,
    ObservableUpDownCounter,
    UpDownCounter,
)
from opentelemetry.sdk.metrics import Histogram as HistogramInstrument
from opentelemetry.sdk.metrics.export import (
    AggregationTemporality,
    DataT,
    Gauge,
    Histogram,
    HistogramDataPoint,
    Metric,
    MetricReader,
    MetricsData,
    Sum,
)
from opentelemetry.sdk.util.instrumentation import InstrumentationScope
from opentelemetry.semconv._incubating.attributes.otel_attributes import (
    OtelComponentTypeValues,
)
from opentelemetry.util.types import Attributes, AttributeValue

_logger = getLogger(__name__)

_TARGET_INFO_NAME = "target"
_TARGET_INFO_DESCRIPTION = "Target metadata"

_OTEL_SCOPE_NAME_LABEL = "otel_scope_name"
_OTEL_SCOPE_VERSION_LABEL = "otel_scope_version"
_OTEL_SCOPE_SCHEMA_URL_LABEL = "otel_scope_schema_url"
_OTEL_SCOPE_ATTR_PREFIX = "otel_scope_"


def _convert_buckets(
    bucket_counts: Sequence[int], explicit_bounds: Sequence[float]
) -> Sequence[tuple[str, int]]:
    buckets = []
    total_count = 0
    for upper_bound, count in zip(
        chain(explicit_bounds, ["+Inf"]),
        bucket_counts,
    ):
        total_count += count
        buckets.append((f"{upper_bound}", total_count))

    return buckets


def _should_convert_sum_to_gauge(metric: Metric) -> bool:
    # The Prometheus compatibility spec requires cumulative non-monotonic Sums
    # to be exported as Gauges.
    if not isinstance(metric.data, Sum):
        return False
    return (
        not metric.data.is_monotonic
        and metric.data.aggregation_temporality
        == AggregationTemporality.CUMULATIVE
    )


_FamilyT = TypeVar("_FamilyT", bound=PrometheusMetric)


def _get_or_create_family(
    registry: dict[str, PrometheusMetric],
    family_id: str,
    factory: Callable[..., _FamilyT],
    *,
    name: str,
    documentation: str,
    labels: Sequence[str],
    unit: str,
) -> _FamilyT:
    if family_id not in registry:
        registry[family_id] = factory(
            name=name,
            documentation=documentation,
            labels=labels,
            unit=unit,
        )
    return registry[family_id]


def _populate_counter_family(
    registry: dict[str, PrometheusMetric],
    per_metric_family_id: str,
    metric_name: str,
    description: str,
    unit: str,
    label_keys: Sequence[str],
    label_rows: Sequence[Sequence[str]],
    values: Sequence[float],
) -> None:
    family_id = "|".join([per_metric_family_id, CounterMetricFamily.__name__])
    family = _get_or_create_family(
        registry,
        family_id,
        CounterMetricFamily,
        name=metric_name,
        documentation=description,
        labels=label_keys,
        unit=unit,
    )
    for label_values, value in zip(label_rows, values):
        family.add_metric(labels=label_values, value=value)


def _populate_gauge_family(
    registry: dict[str, PrometheusMetric],
    per_metric_family_id: str,
    metric_name: str,
    description: str,
    unit: str,
    label_keys: Sequence[str],
    label_rows: Sequence[Sequence[str]],
    values: Sequence[float],
) -> None:
    family_id = "|".join([per_metric_family_id, GaugeMetricFamily.__name__])
    family = _get_or_create_family(
        registry,
        family_id,
        GaugeMetricFamily,
        name=metric_name,
        documentation=description,
        labels=label_keys,
        unit=unit,
    )
    for label_values, value in zip(label_rows, values):
        family.add_metric(labels=label_values, value=value)


def _populate_histogram_family(
    registry: dict[str, PrometheusMetric],
    per_metric_family_id: str,
    metric_name: str,
    description: str,
    unit: str,
    label_keys: Sequence[str],
    label_rows: Sequence[Sequence[str]],
    values: Sequence[dict[str, Any]],
) -> None:
    family_id = "|".join(
        [per_metric_family_id, HistogramMetricFamily.__name__]
    )
    family = _get_or_create_family(
        registry,
        family_id,
        HistogramMetricFamily,
        name=metric_name,
        documentation=description,
        labels=label_keys,
        unit=unit,
    )
    for label_values, value in zip(label_rows, values):
        family.add_metric(
            labels=label_values,
            buckets=_convert_buckets(
                value["bucket_counts"], value["explicit_bounds"]
            ),
            sum_value=value["sum"],
        )


class PrometheusMetricReader(MetricReader):
    """Prometheus metric exporter for OpenTelemetry.

    Args:
        disable_target_info: Whether to disable the ``target_info`` metric.
        scope_info_enabled: Whether to include instrumentation scope labels on
            exported metrics. Scope labels are exported by default.
        prefix: Prefix added to exported Prometheus metric names.
    """

    def __init__(
        self,
        disable_target_info: bool = False,
        prefix: str = "",
        scope_info_enabled: bool = True,
        *,
        registry: CollectorRegistry = REGISTRY,
    ) -> None:
        super().__init__(
            preferred_temporality={
                Counter: AggregationTemporality.CUMULATIVE,
                UpDownCounter: AggregationTemporality.CUMULATIVE,
                HistogramInstrument: AggregationTemporality.CUMULATIVE,
                ObservableCounter: AggregationTemporality.CUMULATIVE,
                ObservableUpDownCounter: AggregationTemporality.CUMULATIVE,
                ObservableGauge: AggregationTemporality.CUMULATIVE,
            },
            otel_component_type=OtelComponentTypeValues.PROMETHEUS_HTTP_TEXT_METRIC_EXPORTER,
        )
        self._collector = _CustomCollector(
            disable_target_info=disable_target_info,
            prefix=prefix,
            scope_info_enabled=scope_info_enabled,
        )
        self._registry = registry
        self._registry.register(self._collector)
        self._collector._callback = self.collect
        self._prefix = prefix

    def _receive_metrics(
        self,
        metrics_data: MetricsData,
        timeout_millis: float = 10_000,
        **kwargs,
    ) -> None:
        if metrics_data is None:
            return
        self._collector.add_metrics_data(metrics_data)

    def shutdown(self, timeout_millis: float = 30_000, **kwargs) -> None:
        self._registry.unregister(self._collector)


class _CustomCollector:
    """_CustomCollector represents the Prometheus Collector object

    See more:
    https://github.com/prometheus/client_python#custom-collectors
    """

    def __init__(
        self,
        disable_target_info: bool = False,
        prefix: str = "",
        scope_info_enabled: bool = True,
    ):
        self._callback = None
        self._metrics_datas: deque[MetricsData] = deque()
        self._disable_target_info = disable_target_info
        self._scope_info_enabled = scope_info_enabled
        self._target_info = None
        self._prefix = prefix

    def add_metrics_data(self, metrics_data: MetricsData) -> None:
        """Add metrics to Prometheus data"""
        self._metrics_datas.append(metrics_data)

    def collect(self) -> Iterable[PrometheusMetric]:
        """Collect fetches the metrics from OpenTelemetry
        and delivers them as Prometheus Metrics.
        Collect is invoked every time a ``prometheus.Gatherer`` is run
        for example when the HTTP endpoint is invoked by Prometheus.
        """
        if self._callback is not None:
            self._callback()

        metric_family_id_metric_family = {}

        if len(self._metrics_datas):
            if not self._disable_target_info:
                if self._target_info is None:
                    attributes: Attributes = {}
                    for res in self._metrics_datas[0].resource_metrics:
                        attributes = {**attributes, **res.resource.attributes}

                    self._target_info = self._create_info_metric(
                        _TARGET_INFO_NAME, _TARGET_INFO_DESCRIPTION, attributes
                    )
                metric_family_id_metric_family[_TARGET_INFO_NAME] = (
                    self._target_info
                )

        while self._metrics_datas:
            self._translate_to_prometheus(
                self._metrics_datas.popleft(), metric_family_id_metric_family
            )

            if metric_family_id_metric_family:
                yield from metric_family_id_metric_family.values()

    def _translate_to_prometheus(
        self,
        metrics_data: MetricsData,
        metric_family_id_metric_family: dict[str, PrometheusMetric],
    ):
        for rm in metrics_data.resource_metrics:
            for sm in rm.scope_metrics:
                scope_attrs = self._build_scope_attrs(sm.scope)
                for metric in sm.metrics:
                    self._translate_metric(
                        metric,
                        scope_attrs,
                        metric_family_id_metric_family,
                    )

    def _translate_metric(
        self,
        metric: Metric,
        scope_attrs: dict[str, Any],
        metric_family_id_metric_family: dict[str, PrometheusMetric],
    ) -> None:
        metric_name = self._resolve_metric_name(metric.name)
        description = metric.description or ""
        unit = map_unit(metric.unit or "")
        label_keys, label_rows, values = self._collect_data_points(
            metric.data, scope_attrs
        )
        per_metric_family_id = "|".join((metric_name, description, unit))

        convert_sum_to_gauge = _should_convert_sum_to_gauge(metric)

        if isinstance(metric.data, Sum) and not convert_sum_to_gauge:
            _populate_counter_family(
                registry=metric_family_id_metric_family,
                per_metric_family_id=per_metric_family_id,
                metric_name=metric_name,
                description=description,
                unit=unit,
                label_keys=label_keys,
                label_rows=label_rows,
                values=values,
            )
        elif isinstance(metric.data, Gauge) or convert_sum_to_gauge:
            _populate_gauge_family(
                registry=metric_family_id_metric_family,
                per_metric_family_id=per_metric_family_id,
                metric_name=metric_name,
                description=description,
                unit=unit,
                label_keys=label_keys,
                label_rows=label_rows,
                values=values,
            )
        elif isinstance(metric.data, Histogram):
            _populate_histogram_family(
                registry=metric_family_id_metric_family,
                per_metric_family_id=per_metric_family_id,
                metric_name=metric_name,
                description=description,
                unit=unit,
                label_keys=label_keys,
                label_rows=label_rows,
                values=values,
            )
        else:
            _logger.warning("Unsupported metric data. %s", type(metric.data))

    def _build_scope_attrs(
        self, scope: InstrumentationScope
    ) -> dict[str, AttributeValue]:
        if not self._scope_info_enabled:
            return {}
        attrs: dict[str, AttributeValue] = {}
        if scope.attributes:
            for key, value in scope.attributes.items():
                attrs[_OTEL_SCOPE_ATTR_PREFIX + key] = value
        attrs[_OTEL_SCOPE_NAME_LABEL] = scope.name or ""
        attrs[_OTEL_SCOPE_VERSION_LABEL] = scope.version or ""
        attrs[_OTEL_SCOPE_SCHEMA_URL_LABEL] = scope.schema_url or ""
        return attrs

    def _resolve_metric_name(self, name: str) -> str:
        if self._prefix:
            name = self._prefix + "_" + name
        return sanitize_full_name(name)

    def _collect_data_points(
        self,
        metric_data: DataT,
        scope_attrs: dict[str, AttributeValue],
    ) -> tuple[list[str], list[list[str]], list[float | dict[str, Any]]]:
        keys: set[str] = set()
        rows: list[dict[str, str]] = []
        values: list[float | dict[str, Any]] = []

        for point in metric_data.data_points:
            labels: dict[str, str] = {}
            for key, value in chain(
                scope_attrs.items(),
                point.attributes.items(),
            ):
                label = sanitize_attribute(key)
                keys.add(label)
                labels[label] = self._check_value(value)
            rows.append(labels)

            if isinstance(point, HistogramDataPoint):
                values.append(
                    {
                        "bucket_counts": point.bucket_counts,
                        "explicit_bounds": point.explicit_bounds,
                        "sum": point.sum,
                    }
                )
            else:
                values.append(point.value)

        label_keys = sorted(keys)
        # Backfill missing labels with "" so every data point exposes the
        # full label set expected by the Prometheus family.
        label_rows = [
            [labels.get(k, "") for k in label_keys] for labels in rows
        ]
        return label_keys, label_rows, values

    # pylint: disable=no-self-use
    def _check_value(self, value: int | float | str | Sequence) -> str:
        """Check the label value and return is appropriate representation"""
        if not isinstance(value, str):
            return dumps(value, default=str)
        return str(value)

    def _create_info_metric(
        self, name: str, description: str, attributes: dict[str, str]
    ) -> InfoMetricFamily:
        """Create an Info Metric Family with list of attributes"""
        # sanitize the attribute names according to Prometheus rule
        attributes = {
            sanitize_attribute(key): self._check_value(value)
            for key, value in attributes.items()
        }
        info = InfoMetricFamily(name, description, labels=attributes)
        info.add_metric(labels=list(attributes.keys()), value=attributes)
        return info


class _AutoPrometheusMetricReader(PrometheusMetricReader):
    """Thin wrapper around PrometheusMetricReader used for the opentelemetry_metrics_exporter entry point.

    This allows users to use the prometheus exporter with opentelemetry-instrument. It handles
    starting the Prometheus http server on the the correct port and host.
    """

    def __init__(self) -> None:
        super().__init__()

        # Default values are specified in
        # https://github.com/open-telemetry/opentelemetry-specification/blob/v1.24.0/specification/configuration/sdk-environment-variables.md#prometheus-exporter
        start_http_server(
            port=int(environ.get(OTEL_EXPORTER_PROMETHEUS_PORT, "9464")),
            addr=environ.get(OTEL_EXPORTER_PROMETHEUS_HOST, "localhost"),
        )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2020, OpenTelemetry Authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import opentelemetry.proto.common.v1.common_pb2
import opentelemetry.proto.resource.v1.resource_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _SeverityNumber:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _SeverityNumberEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_SeverityNumber.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    SEVERITY_NUMBER_UNSPECIFIED: _SeverityNumber.ValueType  # 0
    SEVERITY_NUMBER_TRACE: _SeverityNumber.ValueType  # 1
    SEVERITY_NUMBER_TRACE2: _SeverityNumber.ValueType  # 2
    SEVERITY_NUMBER_TRACE3: _SeverityNumber.ValueType  # 3
    SEVERITY_NUMBER_TRACE4: _SeverityNumber.ValueType  # 4
    SEVERITY_NUMBER_DEBUG: _SeverityNumber.ValueType  # 5
    SEVERITY_NUMBER_DEBUG2: _SeverityNumber.ValueType  # 6
    SEVERITY_NUMBER_DEBUG3: _SeverityNumber.ValueType  # 7
    SEVERITY_NUMBER_DEBUG4: _SeverityNumber.ValueType  # 8
    SEVERITY_NUMBER_INFO: _SeverityNumber.ValueType  # 9
    SEVERITY_NUMBER_INFO2: _SeverityNumber.ValueType  # 10
    SEVERITY_NUMBER_INFO3: _SeverityNumber.ValueType  # 11
    SEVERITY_NUMBER_INFO4: _SeverityNumber.ValueType  # 12
    SEVERITY_NUMBER_WARN: _SeverityNumber.ValueType  # 13
    SEVERITY_NUMBER_WARN2: _SeverityNumber.ValueType  # 14
    SEVERITY_NUMBER_WARN3: _SeverityNumber.ValueType  # 15
    SEVERITY_NUMBER_WARN4: _SeverityNumber.ValueType  # 16
    SEVERITY_NUMBER_ERROR: _SeverityNumber.ValueType  # 17
    SEVERITY_NUMBER_ERROR2: _SeverityNumber.ValueType  # 18
    SEVERITY_NUMBER_ERROR3: _SeverityNumber.ValueType  # 19
    SEVERITY_NUMBER_ERROR4: _SeverityNumber.ValueType  # 20
    SEVERITY_NUMBER_FATAL: _SeverityNumber.ValueType  # 21
    SEVERITY_NUMBER_FATAL2: _SeverityNumber.ValueType  # 22
    SEVERITY_NUMBER_FATAL3: _SeverityNumber.ValueType  # 23
    SEVERITY_NUMBER_FATAL4: _SeverityNumber.ValueType  # 24

class SeverityNumber(_SeverityNumber, metaclass=_SeverityNumberEnumTypeWrapper):
    """Possible values for LogRecord.SeverityNumber."""

SEVERITY_NUMBER_UNSPECIFIED: SeverityNumber.ValueType  # 0
SEVERITY_NUMBER_TRACE: SeverityNumber.ValueType  # 1
SEVERITY_NUMBER_TRACE2: SeverityNumber.ValueType  # 2
SEVERITY_NUMBER_TRACE3: SeverityNumber.ValueType  # 3
SEVERITY_NUMBER_TRACE4: SeverityNumber.ValueType  # 4
SEVERITY_NUMBER_DEBUG: SeverityNumber.ValueType  # 5
SEVERITY_NUMBER_DEBUG2: SeverityNumber.ValueType  # 6
SEVERITY_NUMBER_DEBUG3: SeverityNumber.ValueType  # 7
SEVERITY_NUMBER_DEBUG4: SeverityNumber.ValueType  # 8
SEVERITY_NUMBER_INFO: SeverityNumber.ValueType  # 9
SEVERITY_NUMBER_INFO2: SeverityNumber.ValueType  # 10
SEVERITY_NUMBER_INFO3: SeverityNumber.ValueType  # 11
SEVERITY_NUMBER_INFO4: SeverityNumber.ValueType  # 12
SEVERITY_NUMBER_WARN: SeverityNumber.ValueType  # 13
SEVERITY_NUMBER_WARN2: SeverityNumber.ValueType  # 14
SEVERITY_NUMBER_WARN3: SeverityNumber.ValueType  # 15
SEVERITY_NUMBER_WARN4: SeverityNumber.ValueType  # 16
SEVERITY_NUMBER_ERROR: SeverityNumber.ValueType  # 17
SEVERITY_NUMBER_ERROR2: SeverityNumber.ValueType  # 18
SEVERITY_NUMBER_ERROR3: SeverityNumber.ValueType  # 19
SEVERITY_NUMBER_ERROR4: SeverityNumber.ValueType  # 20
SEVERITY_NUMBER_FATAL: SeverityNumber.ValueType  # 21
SEVERITY_NUMBER_FATAL2: SeverityNumber.ValueType  # 22
SEVERITY_NUMBER_FATAL3: SeverityNumber.ValueType  # 23
SEVERITY_NUMBER_FATAL4: SeverityNumber.ValueType  # 24
global___SeverityNumber = SeverityNumber

class _LogRecordFlags:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _LogRecordFlagsEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_LogRecordFlags.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    LOG_RECORD_FLAGS_DO_NOT_USE: _LogRecordFlags.ValueType  # 0
    """The zero value for the enum. Should not be used for comparisons.
    Instead use bitwise "and" with the appropriate mask as shown above.
    """
    LOG_RECORD_FLAGS_TRACE_FLAGS_MASK: _LogRecordFlags.ValueType  # 255
    """Bits 0-7 are used for trace flags."""

class LogRecordFlags(_LogRecordFlags, metaclass=_LogRecordFlagsEnumTypeWrapper):
    """LogRecordFlags represents constants used to interpret the
    LogRecord.flags field, which is protobuf 'fixed32' type and is to
    be used as bit-fields. Each non-zero value defined in this enum is
    a bit-mask.  To extract the bit-field, for example, use an
    expression like:

      (logRecord.flags & LOG_RECORD_FLAGS_TRACE_FLAGS_MASK)
    """

LOG_RECORD_FLAGS_DO_NOT_USE: LogRecordFlags.ValueType  # 0
"""The zero value for the enum. Should not be used for comparisons.
Instead use bitwise "and" with the appropriate mask as shown above.
"""
LOG_RECORD_FLAGS_TRACE_FLAGS_MASK: LogRecordFlags.ValueType  # 255
"""Bits 0-7 are used for trace flags."""
global___LogRecordFlags = LogRecordFlags

@typing_extensions.final
class LogsData(google.protobuf.message.Message):
    """LogsData represents the logs data that can be stored in a persistent storage,
    OR can be embedded by other protocols that transfer OTLP logs data but do not
    implement the OTLP protocol.

    The main difference between this message and collector protocol is that
    in this message there will not be any "control" or "metadata" specific to
    OTLP protocol.

    When new fields are added into this message, the OTLP request MUST be updated
    as well.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESOURCE_LOGS_FIELD_NUMBER: builtins.int
    @property
    def resource_logs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ResourceLogs]:
        """An array of ResourceLogs.
        For data coming from a single resource this array will typically contain
        one element. Intermediary nodes that receive data from multiple origins
        typically batch the data before forwarding further and in that case this
        array will contain multiple elements.
        """
    def __init__(
        self,
        *,
        resource_logs: collections.abc.Iterable[global___ResourceLogs] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["resource_logs", b"resource_logs"]) -> None: ...

global___LogsData = LogsData

@typing_extensions.final
class ResourceLogs(google.protobuf.message.Message):
    """A collection of ScopeLogs from a Resource."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RESOURCE_FIELD_NUMBER: builtins.int
    SCOPE_LOGS_FIELD_NUMBER: builtins.int
    SCHEMA_URL_FIELD_NUMBER: builtins.int
    @property
    def resource(self) -> opentelemetry.proto.resource.v1.resource_pb2.Resource:
        """The resource for the logs in this message.
        If this field is not set then resource info is unknown.
        """
    @property
    def scope_logs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ScopeLogs]:
        """A list of ScopeLogs that originate from a resource."""
    schema_url: builtins.str
    """The Schema URL, if known. This is the identifier of the Schema that the resource data
    is recorded in. Notably, the last part of the URL path is the version number of the
    schema: http[s]://server[:port]/path/<version>. To learn more about Schema URL see
    https://opentelemetry.io/docs/specs/otel/schemas/#schema-url
    This schema_url applies to the data in the "resource" field. It does not apply
    to the data in the "scope_logs" field which have their own schema_url field.
    """
    def __init__(
        self,
        *,
        resource: opentelemetry.proto.resource.v1.resource_pb2.Resource | None = ...,
        scope_logs: collections.abc.Iterable[global___ScopeLogs] | None = ...,
        schema_url: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["resource", b"resource"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["resource", b"resource", "schema_url", b"schema_url", "scope_logs", b"scope_logs"]) -> None: ...

global___ResourceLogs = ResourceLogs

@typing_extensions.final
class ScopeLogs(google.protobuf.message.Message):
    """A collection of Logs produced by a Scope."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SCOPE_FIELD_NUMBER: builtins.int
    LOG_RECORDS_FIELD_NUMBER: builtins.int
    SCHEMA_URL_FIELD_NUMBER: builtins.int
    @property
    def scope(self) -> opentelemetry.proto.common.v1.common_pb2.InstrumentationScope:
        """The instrumentation scope information for the logs in this message.
        Semantically when InstrumentationScope isn't set, it is equivalent with
        an empty instrumentation scope name (unknown).
        """
    @property
    def log_records(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___LogRecord]:
        """A list of log records."""
    schema_url: builtins.str
    """The Schema URL, if known. This is the identifier of the Schema that the log data
    is recorded in. Notably, the last part of the URL path is the version number of the
    schema: http[s]://server[:port]/path/<version>. To learn more about Schema URL see
    https://opentelemetry.io/docs/specs/otel/schemas/#schema-url
    This schema_url applies to the data in the "scope" field and all logs in the
    "log_records" field.
    """
    def __init__(
        self,
        *,
        scope: opentelemetry.proto.common.v1.common_pb2.InstrumentationScope | None = ...,
        log_records: collections.abc.Iterable[global___LogRecord] | None = ...,
        schema_url: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["scope", b"scope"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["log_records", b"log_records", "schema_url", b"schema_url", "scope", b"scope"]) -> None: ...

global___ScopeLogs = ScopeLogs

@typing_extensions.final
class LogRecord(google.protobuf.message.Message):
    """A log record according to OpenTelemetry Log Data Model:
    https://github.com/open-telemetry/oteps/blob/main/text/logs/0097-log-data-model.md
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIME_UNIX_NANO_FIELD_NUMBER: builtins.int
    OBSERVED_TIME_UNIX_NANO_FIELD_NUMBER: builtins.int
    SEVERITY_NUMBER_FIELD_NUMBER: builtins.int
    SEVERITY_TEXT_FIELD_NUMBER: builtins.int
    BODY_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    DROPPED_ATTRIBUTES_COUNT_FIELD_NUMBER: builtins.int
    FLAGS_FIELD_NUMBER: builtins.int
    TRACE_ID_FIELD_NUMBER: builtins.int
    SPAN_ID_FIELD_NUMBER: builtins.int
    EVENT_NAME_FIELD_NUMBER: builtins.int
    time_unix_nano: builtins.int
    """time_unix_nano is the time when the event occurred.
    Value is UNIX Epoch time in nanoseconds since 00:00:00 UTC on 1 January 1970.
    Value of 0 indicates unknown or missing timestamp.
    """
    observed_time_unix_nano: builtins.int
    """Time when the event was observed by the collection system.
    For events that originate in OpenTelemetry (e.g. using OpenTelemetry Logging SDK)
    this timestamp is typically set at the generation time and is equal to Timestamp.
    For events originating externally and collected by OpenTelemetry (e.g. using
    Collector) this is the time when OpenTelemetry's code observed the event measured
    by the clock of the OpenTelemetry code. This field MUST be set once the event is
    observed by OpenTelemetry.

    For converting OpenTelemetry log data to formats that support only one timestamp or
    when receiving OpenTelemetry log data by recipients that support only one timestamp
    internally the following logic is recommended:
      - Use time_unix_nano if it is present, otherwise use observed_time_unix_nano.

    Value is UNIX Epoch time in nanoseconds since 00:00:00 UTC on 1 January 1970.
    Value of 0 indicates unknown or missing timestamp.
    """
    severity_number: global___SeverityNumber.ValueType
    """Numerical value of the severity, normalized to values described in Log Data Model.
    [Optional].
    """
    severity_text: builtins.str
    """The severity text (also known as log level). The original string representation as
    it is known at the source. [Optional].
    """
    @property
    def body(self) -> opentelemetry.proto.common.v1.common_pb2.AnyValue:
        """A value containing the body of the log record. Can be for example a human-readable
        string message (including multi-line) describing the event in a free form or it can
        be a structured data composed of arrays and maps of other values. [Optional].
        """
    @property
    def attributes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[opentelemetry.proto.common.v1.common_pb2.KeyValue]:
        """Additional attributes that describe the specific event occurrence. [Optional].
        Attribute keys MUST be unique (it is not allowed to have more than one
        attribute with the same key).
        The behavior of software that receives duplicated keys can be unpredictable.
        """
    dropped_attributes_count: builtins.int
    flags: builtins.int
    """Flags, a bit field. 8 least significant bits are the trace flags as
    defined in W3C Trace Context specification. 24 most significant bits are reserved
    and must be set to 0. Readers must not assume that 24 most significant bits
    will be zero and must correctly mask the bits when reading 8-bit trace flag (use
    flags & LOG_RECORD_FLAGS_TRACE_FLAGS_MASK). [Optional].
    """
    trace_id: builtins.bytes
    """A unique identifier for a trace. All logs from the same trace share
    the same `trace_id`. The ID is a 16-byte array. An ID with all zeroes OR
    of length other than 16 bytes is considered invalid (empty string in OTLP/JSON
    is zero-length and thus is also invalid).

    This field is optional.

    The receivers SHOULD assume that the log record is not associated with a
    trace if any of the following is true:
      - the field is not present,
      - the field contains an invalid value.
    """
    span_id: builtins.bytes
    """A unique identifier for a span within a trace, assigned when the span
    is created. The ID is an 8-byte array. An ID with all zeroes OR of length
    other than 8 bytes is considered invalid (empty string in OTLP/JSON
    is zero-length and thus is also invalid).

    This field is optional. If the sender specifies a valid span_id then it SHOULD also
    specify a valid trace_id.

    The receivers SHOULD assume that the log record is not associated with a
    span if any of the following is true:
      - the field is not present,
      - the field contains an invalid value.
    """
    event_name: builtins.str
    """A unique identifier of event category/type.
    All events with the same event_name are expected to conform to the same
    schema for both their attributes and their body.

    Recommended to be fully qualified and short (no longer than 256 characters).

    Presence of event_name on the log record identifies this record
    as an event.

    [Optional].
    """
    def __init__(
        self,
        *,
        time_unix_nano: builtins.int = ...,
        observed_time_unix_nano: builtins.int = ...,
        severity_number: global___SeverityNumber.ValueType = ...,
        severity_text: builtins.str = ...,
        body: opentelemetry.proto.common.v1.common_pb2.AnyValue | None = ...,
        attributes: collections.abc.Iterable[opentelemetry.proto.common.v1.common_pb2.KeyValue] | None = ...,
        dropped_attributes_count: builtins.int = ...,
        flags: builtins.int = ...,
        trace_id: builtins.bytes = ...,
        span_id: builtins.bytes = ...,
        event_name: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["body", b"body"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["attributes", b"attributes", "body", b"body", "dropped_attributes_count", b"dropped_attributes_count", "event_name", b"event_name", "flags", b"flags", "observed_time_unix_nano", b"observed_time_unix_nano", "severity_number", b"severity_number", "severity_text", b"severity_text", "span_id", b"span_id", "time_unix_nano", b"time_unix_nano", "trace_id", b"trace_id"]) -> None: ...

global___LogRecord = LogRecord



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from unittest import TestCase
from unittest.mock import patch

from opentelemetry.sdk.environment_variables._internal import (
    parse_boolean_environment_variable,
)


class TestParseBooleanEnvironmentVariable(TestCase):
    def test_unset_returns_default(self):
        for default, expected in (
            (False, False),
            (True, True),
        ):
            with self.subTest(default=default):
                with patch.dict("os.environ", {}, clear=True):
                    self.assertEqual(
                        parse_boolean_environment_variable(
                            "TEST_BOOL", default=default
                        ),
                        expected,
                    )

    def test_valid_values(self):
        for value, expected in (
            ("true", True),
            (" TrUe ", True),
            ("false", False),
            (" FaLsE ", False),
        ):
            with self.subTest(value=value):
                with patch.dict("os.environ", {"TEST_BOOL": value}):
                    self.assertEqual(
                        parse_boolean_environment_variable("TEST_BOOL"),
                        expected,
                    )

    def test_invalid_value_warns_and_returns_default(self):
        for default, expected in (
            (False, False),
            (True, True),
        ):
            with self.subTest(default=default):
                with patch.dict("os.environ", {"TEST_BOOL": "yes"}):
                    with self.assertLogs(
                        "opentelemetry.sdk.environment_variables._internal",
                        level="WARNING",
                    ) as logs:
                        self.assertEqual(
                            parse_boolean_environment_variable(
                                "TEST_BOOL", default=default
                            ),
                            expected,
                        )

                    self.assertIn(
                        "Invalid value for TEST_BOOL",
                        logs.records[0].message,
                    )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from opentelemetry.sdk.trace._sampling_experimental import (
    composable_always_off,
    composite_sampler,
)
from opentelemetry.sdk.trace.id_generator import RandomIdGenerator
from opentelemetry.sdk.trace.sampling import Decision


def test_description():
    assert composable_always_off().get_description() == "ComposableAlwaysOff"


def test_threshold():
    assert (
        composable_always_off()
        .sampling_intent(None, "test", None, {}, None, None)
        .threshold
        == -1
    )


def test_sampling():
    sampler = composite_sampler(composable_always_off())

    num_sampled = 0
    for _ in range(10000):
        res = sampler.should_sample(
            None,
            RandomIdGenerator().generate_trace_id(),
            "span",
            None,
            None,
            None,
            None,
        )
        if res.decision == Decision.RECORD_AND_SAMPLE:
            num_sampled += 1
        assert not res.trace_state

    assert num_sampled == 0



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# pylint: disable=unused-import,import-outside-toplevel,too-many-locals

from opentelemetry.test import TestCase


class TestImport(TestCase):
    def test_import_init(self):
        """
        Test that the metrics root module has the right symbols
        """

        with self.assertNotRaises(Exception):
            from opentelemetry.sdk.metrics import (  # noqa: F401, PLC0415
                Counter,
                Histogram,
                Meter,
                MeterProvider,
                ObservableCounter,
                ObservableGauge,
                ObservableUpDownCounter,
                UpDownCounter,
                _Gauge,
            )

    def test_import_export(self):
        """
        Test that the metrics export module has the right symbols
        """

        with self.assertNotRaises(Exception):
            from opentelemetry.sdk.metrics.export import (  # noqa: F401, PLC0415
                AggregationTemporality,
                ConsoleMetricExporter,
                DataPointT,
                DataT,
                Gauge,
                Histogram,
                HistogramDataPoint,
                InMemoryMetricReader,
                Metric,
                MetricExporter,
                MetricExportResult,
                MetricReader,
                MetricsData,
                NumberDataPoint,
                PeriodicExportingMetricReader,
                ResourceMetrics,
                ScopeMetrics,
                Sum,
            )

    def test_import_view(self):
        """
        Test that the metrics view module has the right symbols
        """

        with self.assertNotRaises(Exception):
            from opentelemetry.sdk.metrics.view import (  # noqa: F401, PLC0415
                Aggregation,
                DefaultAggregation,
                DropAggregation,
                ExplicitBucketHistogramAggregation,
                LastValueAggregation,
                SumAggregation,
                View,
            )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0
import collections.abc
import logging

from typing_extensions import deprecated

from opentelemetry.context.context import Context
from opentelemetry.propagators import textmap

logger = logging.getLogger(__name__)


class CompositePropagator(textmap.TextMapPropagator):
    """CompositePropagator provides a mechanism for combining multiple
    propagators into a single one.

    Args:
        propagators: the list of propagators to use
    """

    def __init__(
        self, propagators: collections.abc.Sequence[textmap.TextMapPropagator]
    ) -> None:
        self._propagators = propagators

    def extract(
        self,
        carrier: textmap.CarrierT,
        context: Context | None = None,
        getter: textmap.Getter[textmap.CarrierT] = textmap.default_getter,
    ) -> Context:
        """Run each of the configured propagators with the given context and carrier.
        Propagators are run in the order they are configured, if multiple
        propagators write the same context key, the propagator later in the list
        will override previous propagators.

        See `opentelemetry.propagators.textmap.TextMapPropagator.extract`
        """
        for propagator in self._propagators:
            context = propagator.extract(carrier, context, getter=getter)
        return context  # type: ignore

    def inject(
        self,
        carrier: textmap.CarrierT,
        context: Context | None = None,
        setter: textmap.Setter[textmap.CarrierT] = textmap.default_setter,
    ) -> None:
        """Run each of the configured propagators with the given context and carrier.
        Propagators are run in the order they are configured, if multiple
        propagators write the same carrier key, the propagator later in the list
        will override previous propagators.

        See `opentelemetry.propagators.textmap.TextMapPropagator.inject`
        """
        for propagator in self._propagators:
            propagator.inject(carrier, context, setter=setter)

    @property
    def fields(self) -> set[str]:
        """Returns a set with the fields set in `inject`.

        See
        `opentelemetry.propagators.textmap.TextMapPropagator.fields`
        """
        composite_fields = set()

        for propagator in self._propagators:
            for field in propagator.fields:
                composite_fields.add(field)

        return composite_fields


@deprecated(
    "You should use CompositePropagator. Deprecated since version 1.2.0."
)
class CompositeHTTPPropagator(CompositePropagator):
    """CompositeHTTPPropagator provides a mechanism for combining multiple
    propagators into a single one.
    """



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from opentelemetry import metrics, trace
from opentelemetry.exporter.otlp.proto.grpc.metric_exporter import (
    OTLPMetricExporter,
)
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import (
    OTLPSpanExporter,
)
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import PeriodicExportingMetricReader
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor

bind = "127.0.0.1:8000"

# Sample Worker processes
workers = 4
worker_class = "sync"
worker_connections = 1000
timeout = 30
keepalive = 2

# Sample logging
errorlog = "-"
loglevel = "info"
accesslog = "-"
access_log_format = (
    '%(h)s %(l)s %(u)s %(t)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"'
)


def post_fork(server, worker):
    server.log.info("Worker spawned (pid: %s)", worker.pid)

    resource = Resource.create(
        attributes={
            "service.name": "api-service",
            # If workers are not distinguished within attributes, traces and
            # metrics exported from each worker will be indistinguishable. While
            # not necessarily an issue for traces, it is confusing for almost
            # all metric types. A built-in way to identify a worker is by PID
            # but this may lead to high label cardinality. An alternative
            # workaround and additional discussion are available here:
            # https://github.com/benoitc/gunicorn/issues/1352
            "worker": worker.pid,
        }
    )

    trace.set_tracer_provider(TracerProvider(resource=resource))
    # This uses insecure connection for the purpose of example. Please see the
    # OTLP Exporter documentation for other options.
    span_processor = BatchSpanProcessor(
        OTLPSpanExporter(endpoint="http://localhost:4317", insecure=True)
    )
    trace.get_tracer_provider().add_span_processor(span_processor)

    reader = PeriodicExportingMetricReader(
        OTLPMetricExporter(endpoint="http://localhost:4317")
    )
    metrics.set_meter_provider(
        MeterProvider(
            resource=resource,
            metric_readers=[reader],
        )
    )
